#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_actuals import (TuneV2AdvertiserStatsActuals)
from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_base import (
    TuneV2AdvertiserStatsActions, TuneV2AdvertiserStatsFormats
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""Report Reader JSON
"""

import ujson as json
from logging import getLogger
import requests
# from pprintpp import pprint

from .report_reader_base import (ReportReaderBase)

log = getLogger(__name__)


# @brief TUNE Multiverse Report JSON reader.
#
# @namespace tune_reporting.ReportReaderCSV
class ReportReaderJSON(ReportReaderBase):
    """Helper class for reading reading remote JSON file"""

    #  The constructor
    #  @param str report_url Download report URL
    #                         of requested report to be exported.
    def __init__(self, report_url):
        """The constructor.

            :param str report_url: Report URL to be downloaded.
        """
        super(ReportReaderJSON, self).__init__(report_url)

    #  Using provided report download URL, extract JSON contents.
    #
    def read(self):
        """Read JSON data provided remote path report_url."""
        self.data = None

        response = requests.get(url=self.report_url)

        log.info("ReportReaderJSON: Response", extra={'http_status_code': response.status_code})

        self.data = json.loads(response.text)
        self.count = len(self.data)

    def pretty_print(self, limit=0):
        """Pretty print exported data.

            :param int limit: Number of rows to print.
        """
        print("Report REPORT_URL: {}".format(self.report_url))
        print("Report total row count: {}".format(self.count))
        if self.count > 0:
            print("------------------")
            rows = list(self.data)
            i = 0
            for row in enumerate(rows):
                i = i + 1
                print("{}. {}".format(i, str(row)))
                if (limit > 0) and (i >= limit):
                    break
            print("------------------")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertiser Stats Abstract Base Class.
"""

import datetime as dt
import logging
import os
import time
from abc import ABCMeta, abstractmethod
from urllib.parse import urlparse
from pyhttpstatus_utils import (
    is_http_status_type,
    HttpStatusType,
)
from requests_mv_integrations.support import (validate_json_response)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (
    print_traceback,
    get_exception_message,
    TuneReportingErrorCodes,
)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import python_check_version
from tune_reporting.readers.report_reader_json import (ReportReaderJSON)
from tune_reporting import (__python_required_version__)
from tune_reporting.readers.report_reader_csv import (ReportReaderCSV)
from tune_reporting.tmc.tune_mobileapptracking_api import (TuneMobileAppTrackingApi)
from tune_reporting.tmc.v2.management import (TuneV2AuthenticationTypes)
from tune_reporting.tmc.tmc_auth_v2_session_token import (tmc_auth_v2_session_token)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)
from safe_cast import (
    safe_int,
    safe_dict,
)

python_check_version(__python_required_version__)


# @brief TUNE v3 Logs Actions ENUM
#
# @namespace tune_reporting.TuneV3LogsAdvertisersActions
class TuneV3LogsAdvertisersActions(object):
    """v3 Logs Actions
    """
    FIND = "find"
    EXPORT = "export"


# @brief TUNE Advertiser Stats Formats ENUM
#
# @namespace tune_reporting.TuneV3LogsAdvertisersLimitMax
class TuneV3LogsAdvertisersLimitMax(object):
    """Data formats for Advertiser Sites.
    """
    FIND = 5000
    EXPORT = 200000


# @brief TUNE Advertiser Stats Formats ENUM
#
# @namespace tune_reporting.TuneV3LogsAdvertisersResponseFormats
class TuneV3LogsAdvertisersResponseFormats(object):
    """Data formats for Advertiser Sites.
    """
    JSON = "json"
    CSV = "csv"


# @brief TUNE Advertiser Stats Abstract Base Class.
#
# @namespace tune_reporting.TuneV3LogsAdvertisersBase
class TuneV3LogsAdvertisersBase(TuneMobileAppTrackingApi):
    """TUNE Advertiser Stats Abstract Base Class.
    """

    __metaclass__ = ABCMeta

    _FILTER_NOT_DEBUG_NOR_TEST_DATA = (
        "(debug_mode = 0 OR debug_mode is NULL)"
        " AND "
        "(test_profile_id = 0 OR test_profile_id IS NULL)"
    )

    _CONTROLLER = "logs/advertisers/{advertiser_id}"

    # Initialize Job
    #
    def __init__(
        self,
        logs_advertisers_type,
        timezone=None,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR
    ):
        """Initialize TUNE Advertiser Stats Abstract Base Class.
        """
        super(TuneV3LogsAdvertisersBase, self).__init__(
            timezone=timezone,
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

        self.controller = None
        self.logs_advertisers_type = logs_advertisers_type

    # Collect data: TUNE Advertiser Stats Actuals.
    #
    def collect(
        self,
        auth_value,
        auth_type,
        auth_type_use,
        start_date,
        end_date,
        advertiser_id,
        request_params=None,
        request_retry=None,
        request_action=TuneV3LogsAdvertisersActions.FIND
    ):
        """Collect data: TUNE Advertiser Stats Actuals.

        Args:
            tmc_api_key:
            start_date:
            end_date:
            advertiser_id:
            request_params:
            request_retry:
            request_action:

        Returns:

        """
        if not auth_value:
            raise ValueError("TMC v3 Logs Advertisers Base: Collect: Value 'auth_value' not provided.")
        if not auth_type:
            raise ValueError("TMC v3 Logs Advertisers Base: Collect: Value 'auth_type' not valid.")
        if not auth_type_use or \
                not TuneV2AuthenticationTypes.validate(auth_type_use):
            raise ValueError("TMC v3 Logs Advertisers Base: Collect: Value 'auth_type_use' not valid.")
        if not advertiser_id:
            raise ValueError("TMC v3 Logs Advertisers Base: Collect: Value 'advertiser_id' not valid.")

        if auth_type == TuneV2AuthenticationTypes.API_KEY:
            self.api_key = auth_value

            if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
                self.session_token = tmc_auth_v2_session_token(
                    tmc_api_key=self.api_key,
                    logger_level=self.logger_level,
                    logger_format=self.logger_format,

                )
        elif auth_type == TuneV2AuthenticationTypes.SESSION_TOKEN:
            self.session_token = auth_value
        else:
            raise ValueError(error_message="Invalid 'auth_type': '{}'".format(auth_type))

        auth_value_use = None
        if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            if self.session_token is None:
                raise TuneReportingError(
                    error_message="Value 'session_token' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
                )

            auth_value_use = self.session_token

        elif auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            if self.api_key is None:
                raise TuneReportingError(
                    error_message="Value 'api_key' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE,
                )

            auth_value_use = self.api_key

        if not auth_value_use:
            raise TuneReportingError(
                error_message="Value 'auth_value_use' not defined.",
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE,
            )

        self.advertiser_id = advertiser_id

        self.controller = self._CONTROLLER.format(advertiser_id=self.advertiser_id)

        self._collect(auth_type_use, start_date, end_date, request_params, request_retry, request_action)

    @abstractmethod
    def _collect(
        self,
        auth_type_use,
        start_date,
        end_date,
        request_params=None,
        request_retry=None,
        request_action=TuneV3LogsAdvertisersActions.FIND
    ):
        """Collect data: TUNE Advertiser Stats.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:
            request_action:

        Returns:

        """

        raise NotImplementedError

    def _find_v3(self, request_params, request_retry=None, request_label="TMC v3 Logs Advertisers"):
        """Gather data using action 'find'

        Args:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.debug(("TMC v3 Logs Advertisers Base: "
                           "Logs '{}': "
                           "Action 'find'").format(self.logs_advertisers_type))

        if "start_date" not in request_params:
            raise ValueError("Missing attribute 'start_date' in parameter 'request_params'")
        request_start_date = request_params["start_date"]

        if "end_date" not in request_params:
            raise ValueError("Missing attribute 'end_date' in parameter 'request_params'")
        request_end_date = request_params["end_date"]

        datetime_start = dt.datetime.strptime(request_start_date, '%Y-%m-%d')
        datetime_end = dt.datetime.strptime(request_end_date, '%Y-%m-%d')

        str_date_start = str(datetime_start.date())
        str_date_end = str(datetime_end.date())

        self.logger.debug(
            "TMC v3 Logs Advertisers Base: Action 'find'",
            extra={'start_date': str_date_start,
                   'end_date': str_date_end}
        )

        if not request_retry:
            request_retry = {'timeout': 600}

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v3",
                controller=self.controller,
                action=self.logs_advertisers_type
            )

        if 'response_format' not in request_params:
            request_params['response_format'] = \
                TuneV3LogsAdvertisersResponseFormats.JSON

        request_params["start_date"] += "T00:00:00Z"
        request_params["end_date"] += "T23:59:59Z"

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_retry=None,
                request_retry_http_status_codes=None,
                request_retry_func=None,
                request_retry_excps_func=None,
                request_label=request_label
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Base: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Base: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TMC v3 Logs Advertisers Base: {}".format(get_exception_message(ex)))
            raise

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v3 Logs Advertisers: Action 'find'",
        )

        data = json_response['data']

        self.data = data

    def _export_v3_download_csv(self, request_params, request_retry=None):
        """Export and then Read download CSV

        Args:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.debug(("TMC v3 Logs Advertisers Base: "
                           "Logs '{}': "
                           "Action 'exports'").format(self.logs_advertisers_type))

        data = []

        delta_day = dt.timedelta(days=1)

        if "start_date" not in request_params:
            raise ValueError("Missing attribute 'start_date' in parameter 'request_params'")
        request_start_date = request_params["start_date"]

        if "end_date" not in request_params:
            raise ValueError("Missing attribute 'end_date' in parameter 'request_params'")
        request_end_date = request_params["end_date"]

        datetime_start = dt.datetime.strptime(request_start_date, '%Y-%m-%d')
        datetime_end = dt.datetime.strptime(request_end_date, '%Y-%m-%d')

        str_date_start = str(datetime_start.date())
        str_date_end = str(datetime_end.date())

        self.logger.debug(
            msg=("TuneV3LogsAdvertisersBase"),
            extra={'action': 'export',
                   'start_date': str_date_start,
                   'end_date': str_date_end}
        )

        next_end_date = datetime_start - delta_day

        done = False

        export_jobs = []

        while not done:
            next_start_date = next_end_date + delta_day
            next_end_date = next_end_date + delta_day

            if next_end_date > datetime_end:
                next_end_date = datetime_end

            if next_end_date == datetime_end:
                done = True

            # pylint: disable=no-member
            next_start_date_yyyy_mm_dd = \
                next_start_date.strftime('%Y-%m-%d')

            next_end_date_yyyy_mm_dd = \
                next_end_date.strftime('%Y-%m-%d')
            # pylint: enable=no-member

            next_start_date_yyyy_mm_dd += "T00:00:00Z"
            next_end_date_yyyy_mm_dd += "T23:59:59Z"

            request_params["start_date"] = \
                next_start_date_yyyy_mm_dd
            request_params["end_date"] = \
                next_end_date_yyyy_mm_dd

            export_job = self._export_v3_job_to_queue(request_params, request_retry)

            self.logger.debug(
                "TuneV3LogsAdvertisersBase",
                extra={
                    'action': 'export',
                    'start_date': next_start_date_yyyy_mm_dd,
                    'end_date': next_end_date_yyyy_mm_dd,
                    'job': export_job
                }
            )

            export_jobs.append({
                'job_handle': export_job,
                'start_date': next_start_date_yyyy_mm_dd,
                'end_date': next_end_date_yyyy_mm_dd
            })

        self.logger.debug("TuneV3LogsAdvertisersBase", extra={'action': 'export', 'export_job_count': len(export_jobs)})

        jobs_total_count = 0

        for export_job in export_jobs:
            export_job_handle = export_job['job_handle']
            export_job_start_date = export_job['start_date']
            export_job_end_date = export_job['end_date']

            report_url = self._check_v3_job_status_on_queue(export_job_handle, request_retry=request_retry)

            generator_data_stats = self._read_stats_data(export_job_handle, report_url)

            job_row_count = 0

            for row in list(generator_data_stats):
                if len(row) == 0:
                    continue
                job_row_count += 1
                data.append(row)

            jobs_total_count += job_row_count

            self.logger.debug(
                msg=("TuneV3LogsAdvertisersBase"),
                extra={
                    'action': 'export',
                    'start_date': export_job_start_date,
                    'end_date': export_job_end_date,
                    'job_handle': export_job_handle,
                    'job_row_count': job_row_count
                }
            )

        self.logger.debug(
            msg=("TuneV3LogsAdvertisersBase"),
            extra={'action': 'export',
                   'total_data_count': len(data),
                   'total_job_sum_count': jobs_total_count}
        )

        self.data = data

    def _export_v3_job_to_queue(self, request_params, request_retry=None, request_label="TMC v3 Job On Queue"):
        """Gather data using action export.json.

        :param request_params:
        :param request_retry:
        :return:
        """

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v3",
                controller=self.controller,
                action="exports/{}".format(
                    self.logs_advertisers_type
                )
            )

        self.logger.debug(
            msg=("TMC v3 Logs Advertisers Base: "
                 "Logs '{}': "
                 "Action 'exports': "
                 "Start Advertiser Report Actuals").format(self.logs_advertisers_type),
            extra={'start_date': request_params["start_date"],
                   'end_date': request_params["end_date"]}
        )

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_retry=request_retry,
                request_retry_http_status_codes=None,
                request_retry_func=self.tune_v3_request_retry_func,
                request_retry_excps_func=None,
                request_label=request_label
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Base: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Base: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TMC v3 Logs Advertisers Base: {}".format(get_exception_message(ex)))
            raise

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v3 Logs Advertisers: '{}': Action 'exports'".format(self.logs_advertisers_type)
        )

        if ('handle' not in json_response or not json_response['handle']):
            raise TuneReportingError(
                error_message=(
                    "TMC v3 Logs Advertisers Base: "
                    "Logs '{}': "
                    "Action 'exports': "
                    "Response missing 'handle': {}"
                ).format(self.logs_advertisers_type, str(json_response)),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )

        export_job = json_response['handle']

        if not export_job:
            raise TuneReportingError(
                error_message=(
                    "TMC v3 Logs Advertisers Base: "
                    "Logs '{}': "
                    "Action 'exports': "
                    "Response missing 'handle'"
                ).format(self.logs_advertisers_type),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )

        self.logger.info(
            "TuneV3LogsAdvertisersBase",
            extra={
                'advertiser_type': self.logs_advertisers_type,
                'action': 'exports',
                'start_date': request_params["start_date"],
                'end_date': request_params["end_date"],
                'job': export_job
            }
        )

        return export_job

    def _check_v3_job_status_on_queue(self, export_job, request_retry=None, request_label="TMC v3 Job Status On Queue"):
        """Status of Export Report.

        Args:
            export_job:
            request_retry:

        Returns:

        """
        request_label = "v3 Logs Advertisers Check Export Status"

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v3",
                controller=self.controller,
                action="exports/{}".format(
                    export_job
                )
            )

        self.logger.info((
            "TMC v3 Logs Advertisers Base: Check Export Status: "
            "Logs '{}': "
            "Action: 'exports status', "
            "Status of Export Report for "
            "Job Handle: '{}'"
        ).format(self.logs_advertisers_type, export_job))

        tries = -1  # default: -1 (indefinite)
        delay = 10
        jitter = 0
        max_delay = 60

        if request_retry:
            if 'delay' in request_retry:
                delay = request_retry['delay']
            if 'jitter' in request_retry:
                jitter = request_retry['jitter']
            if 'max_delay' in request_retry:
                max_delay = request_retry['max_delay']

        request_params = {"session_token": self.session_token}

        self.logger.debug("TMC v3 Logs Advertisers Base: Check Export Status", extra={'request_url': request_url})

        self.logger.debug(
            "TMC v3 Logs Advertisers Base: Check Export Status: Request Retry",
            extra={'tries': tries,
                   'delay': delay,
                   'jitter': jitter,
                   'max_delay': max_delay}
        )

        self.logger.debug(
            "TMC v3 Logs Advertisers Base: Check Export Status: Request",
            extra={'request_params': safe_dict(request_params)}
        )

        report_url = None
        _attempts = 1
        export_percent_complete = 0

        export_status_action = 'exports status'

        self.logger.warning(
            "TMC v3 Logs Advertisers Base: Check Export Status",
            extra={'job': export_job,
                   'attempt': _attempts,
                   'action': export_status_action}
        )

        time.sleep(10)

        _tries, _delay = tries, delay
        while True:
            try:
                response = self.mv_request.request(
                    request_method="GET",
                    request_url=request_url,
                    request_params=request_params,
                    request_retry=None,
                    request_retry_http_status_codes=None,
                    request_retry_func=self.tune_v3_request_retry_func,
                    request_retry_excps_func=None,
                    request_label=request_label
                )

            except TuneRequestBaseError as tmc_req_ex:
                self.logger.error(
                    "TMC v3 Logs Advertisers Base: Check Export Status: Failed",
                    extra=tmc_req_ex.to_dict(),
                )
                raise

            except TuneReportingError as tmc_rep_ex:
                self.logger.error(
                    "TMC v3 Logs Advertisers Base: Check Export Status: Failed",
                    extra=tmc_rep_ex.to_dict(),
                )
                raise

            except Exception as ex:
                print_traceback(ex)

                self.logger.error(
                    "TMC v3 Logs Advertisers Base: Check Export Status: Failed",
                    extra={'error': get_exception_message(ex)}
                )
                raise

            http_status_successful = is_http_status_type(
                http_status_code=response.status_code, http_status_type=HttpStatusType.SUCCESSFUL
            )

            if not http_status_successful:
                raise TuneReportingError(
                    error_message="Failed to get export status on queue: {}".format(response.status_code),
                    error_code=TuneReportingErrorCodes.REP_ERR_REQUEST
                )

            json_response = response.json()

            export_percent_complete = 0
            if "percent_complete" in json_response:
                export_percent_complete = \
                    safe_int(json_response["percent_complete"])

            self.logger.info(
                "TMC v3 Logs Advertisers Base: Check Job Export Status",
                extra={
                    'job': export_job,
                    'response_status_code': json_response["status"],
                    'export_percent_complete': safe_int(export_percent_complete)
                }
            )

            if (export_percent_complete == 100 and json_response["status"] == "complete" and json_response["url"]):
                report_url = json_response["url"]

                self.logger.info(
                    "TMC v3 Logs Advertisers Base: Check Job Export Status: Completed",
                    extra={
                        'job': export_job,
                        'report_url': report_url,
                        'request_label': request_label,
                        'export_percent_complete': safe_int(export_percent_complete)
                    }
                )

                break

            if tries >= 0:
                _tries -= 1
                if _tries == 0:
                    self.logger.error(("TMC v3 Logs Advertisers Base: "
                                       "Check Job Export Status: Exhausted Retries"),
                                      extra={
                                          'attempt': _attempts,
                                          'tries': _tries,
                                          'action': export_status_action,
                                          'request_label': request_label,
                                          'export_percent_complete': export_percent_complete
                                      })

                    raise TuneReportingError(
                        error_message=(
                            "TMC v3 Logs Advertisers Base: "
                            "Check Job Export Status: "
                            "Exhausted Retries: "
                            "Percent Completed: {}"
                        ).format(export_percent_complete),
                        error_code=TuneReportingErrorCodes.REP_ERR_JOB_STOPPED
                    )

            _attempts += 1

            self.logger.warning(
                "TMC v3 Logs Advertisers Base: Check Export Status",
                extra={'attempt': _attempts,
                       'job': export_job,
                       'delay': _delay,
                       'action': 'exports status'}
            )
            time.sleep(_delay)

            _delay += jitter
            _delay = min(_delay, max_delay)

        if export_percent_complete == 100 and not report_url:
            raise TuneReportingError(
                error_message=(
                    "TMC v3 Logs Advertisers Base: Check Job Export Status: "
                    "Download report URL: Undefined"
                )
            )

        self.logger.info(
            "TMC v3 Logs Advertisers Base: Check Job Export Status: Finished",
            extra={
                'attempt': _attempts,
                'action': export_status_action,
                'report_url': report_url,
                'request_label': request_label,
                'export_percent_complete': export_percent_complete,
                'job': export_job
            }
        )

        return report_url

    def _read_stats_data(self, export_job_handle, report_url, report_format=None):
        if not export_job_handle:
            raise ValueError("Argument 'export_job_handle' not assigned.")
        if not report_url:
            raise ValueError("Argument 'report_url' not assigned.")

        self.logger.info(("TMC v3 Logs Advertisers Base: "
                          "Start Advertiser Report Stats read: "
                          "Job Handle: '{}'").format(export_job_handle))

        if not report_format:
            path = urlparse(report_url).path
            ext = os.path.splitext(path)[1]
            report_format = ext[1:]

        self.logger.info(("TMC v3 Logs Advertisers Base: "
                          "Reading URL '{}' with format '{}'").format(report_url, report_format))

        # pylint: disable=redefined-variable-type
        if report_format == TuneV3LogsAdvertisersResponseFormats.JSON:
            report_reader = ReportReaderJSON(report_url)
        elif report_format == TuneV3LogsAdvertisersResponseFormats.CSV:
            report_reader = ReportReaderCSV(report_url)
        else:
            raise TuneReportingError(error_message="Unexpected report format: '{}'".format(report_format))
        # pylint: enable=redefined-variable-type

        if not report_reader:
            raise TuneReportingError(error_message="Report reader not created for format {}".format(report_format))

        report_reader.read()

        self.logger.info(("TMC v3 Logs Advertisers Base: " "Finished Advertiser Report Stats read"))

        return report_reader.generator

    def tune_v3_request_retry_func(self, response):
        """Request Retry Function

        Args:
            response:

        Returns:
            Boolean

        """
        response_json = response.json()

        self.logger.debug("Check for Retry", extra={'response': response_json})

        tune_v2_errors = None
        tune_v2_errors_error_code = None
        tune_v2_errors_message = None

        if 'errors' in response_json:
            tune_v2_errors = response_json['errors']
            tune_v2_errors_error_code = tune_v2_errors[0]['error_code']
            tune_v2_errors_message = tune_v2_errors[0]['message']

        self.logger.debug(
            "Check for Retry: Response: Status: {}, Errors: '{}'".
            format(tune_v2_errors_error_code, tune_v2_errors_message)
        )

        if tune_v2_errors is None:
            return False

        # For V3, it is not clear what would be a retry status.
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE MobileAppTracking API base class
"""

import logging

from pyhttpstatus_utils import (HttpStatusType, is_http_status_type)
from requests_mv_integrations.support import (validate_json_response)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (print_traceback, get_exception_message, TuneReportingErrorCodes)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)
from tune_reporting import (__python_required_version__)
from tune_reporting.tmc.tune_mobileapptracking_api_base import (TuneMobileAppTrackingApiBase)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)

python_check_version(__python_required_version__)


# @brief TUNE Authentication Types ENUM
#
# @namespace tune_reporting.TuneV2AuthenticationTypes
class TuneV2AuthenticationTypes(object):
    """TUNE Authentication Types ENUM
    """
    API_KEY = "api_key"
    SESSION_TOKEN = "session_token"

    @staticmethod
    def validate(auth_type):
        return auth_type in [
            TuneV2AuthenticationTypes.API_KEY,
            TuneV2AuthenticationTypes.SESSION_TOKEN,
        ]


# @brief  TUNE MobileAppTracking API v2/session/authenticate
#
# @namespace tune_reporting.TuneV2SessionAuthenticate
class TuneV2SessionAuthenticate(TuneMobileAppTrackingApiBase):
    """TUNE MobileAppTracking API v2/session/authenticate
    """

    # Initialize Job
    #
    def __init__(
        self,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR
    ):
        super(TuneV2SessionAuthenticate, self).__init__(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

    def get_session_token(self, tmc_api_key, request_retry=None):
        """Generate session token is returned to provide
        access to service.

        Args:
            tmc_api_key:
            request_retry:

        Returns:

        """
        self.logger.info("TMC v2 Session Authenticate: Get Token")

        self.api_key = tmc_api_key

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller="session/authenticate",
                action="api_key"
            )

        request_params = \
            {
                'api_keys': tmc_api_key,
                "source": "multiverse"
            }

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_retry=None,
                request_retry_http_status_codes=None,
                request_retry_func=self.tune_v2_request_retry_func,
                request_retry_excps_func=None,
                request_label="TMC v2 Session Authenticate"
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v2 Session Authenticate: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v2 Session Authenticate: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TMC v2 Session Authenticate: Failed: {}".format(get_exception_message(ex)))

            raise TuneReportingError(
                error_message=("TMC v2 Session Authenticate: Failed: {}").format(get_exception_message(ex)),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v2 Get Session Token: Validate",
        )

        self.logger.debug(
            "TMC v2 Session Authenticate: Details:",
            extra={'response': json_response},
        )

        response_status_code = json_response.get('status_code', None)
        status_code_successful = is_http_status_type(
            http_status_code=response_status_code, http_status_type=HttpStatusType.SUCCESSFUL
        )

        response_errors = json_response.get('errors', None)

        response_error_message = ""
        if response_errors:
            if isinstance(response_errors, dict):
                error_message = response_errors.get('message', None)
                if error_message:
                    if error_message.startswith("Invalid api key"):
                        response_status_code = TuneReportingErrorCodes.UNAUTHORIZED
                    response_error_message += error_message
            elif isinstance(response_errors, list):
                for response_error in response_errors:
                    if isinstance(response_error, dict) \
                            and 'message' in response_error:
                        error_message = response_error.get('message', None)
                        if error_message:
                            if error_message.startswith("Invalid api key"):
                                response_status_code = TuneReportingErrorCodes.UNAUTHORIZED
                            response_error_message += error_message

        error_code = response_status_code

        if not status_code_successful or not json_response['data']:
            raise TuneReportingError(
                error_message="TMC v2 Session Authenticate: Failed: {}".format(response_error_message),
                error_code=error_code,
            )

        self.session_token = json_response['data']

        self.logger.info("TMC v2 Session Authenticate", extra={'session_token': self.session_token})
        self.logger.info("TMC v2 Session Authenticate: Finished")

        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @namespace tune_reporting

from .tmc_v2_advertiser_sites import (TuneV2AdvertiserSites, TuneV2AdvertiserSiteStatus)
from .tmc_v2_advertisers import (TuneV2Advertisers)
from .tmc_v2_session_authenticate import (TuneV2SessionAuthenticate, TuneV2AuthenticationTypes)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace requests_mv_integrations

import copy
import re
import ujson as json
import urllib.parse
from .constants import (
    HEADER_CONTENT_TYPE_APP_JSON,
    __USER_AGENT__,
)


def command_line_request_curl(
    request_method,
    request_url,
    request_headers,
    request_data=None,
    request_json=None,
    request_timeout=60,
    request_allow_redirects=True
):
    """Command Line: Build Request cUrl

    Args:
        request_method:
        request_url:
        request_headers:
        request_data:
        request_timeout:
        request_allow_redirects:

    Returns:

    """
    key_user_agent = 'User-Agent'
    header_user_agent = {key_user_agent: __USER_AGENT__}

    if request_headers:
        if key_user_agent not in request_headers:
            request_headers.update(header_user_agent)
    else:
        request_headers = header_user_agent

    request_method = request_method.upper()

    command = ("curl" " --verbose" " -X {request_method}" " -H {headers}" " --connect-timeout {timeout}")

    if request_allow_redirects:
        command += " -L"

    if request_method == 'GET':
        if request_data:
            params = request_data.split("&")

            command += (" -G" " --data {params}" " '{url}'")

            params = ["'{0}'".format(urllib.parse.unquote(param)) for param in params]
            params = " --data ".join(params)

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)

            return command.format(
                request_method=request_method, headers=headers, params=params, timeout=request_timeout, url=request_url
            )
        else:
            command += (" '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)

            return command.format(
                request_method=request_method, headers=headers, timeout=request_timeout, url=request_url
            )

    elif request_method == 'POST':
        if request_data:
            command += (" --data '{data}'" " '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)

            return command.format(
                request_method=request_method,
                headers=headers,
                data=request_data,
                timeout=request_timeout,
                url=request_url
            )
        elif request_json:
            command += (" --data '{data}'" " '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)

            return command.format(
                request_method=request_method,
                headers=headers,
                data=json.dumps(request_json),
                timeout=request_timeout,
                url=request_url
            )
        else:
            command += (" '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)
            return command.format(
                request_method=request_method, headers=headers, timeout=request_timeout, url=request_url
            )

    elif request_method == 'PUT':
        if request_data:
            rows = re.split(r'\n', request_data)
            row = None
            if rows and len(rows) > 0:
                row = rows[0]

            command += (" --data '{data}'" " '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)

            return command.format(
                request_method=request_method, headers=headers, data=row, timeout=request_timeout, url=request_url
            )
        else:
            command += (" '{url}'")

            headers = ["'{0}: {1}'".format(k, v) for k, v in request_headers.items()]
            headers = " -H ".join(headers)
            return command.format(
                request_method=request_method, headers=headers, timeout=request_timeout, url=request_url
            )


def command_line_request_curl_get(
    request_url, request_params, request_headers=None, request_timeout=60, request_allow_redirects=True
):
    """Command Line: Build Request cUrl for GET method

    Args:
        request_url:
        request_params:
        request_headers:
        request_timeout:
        request_allow_redirects:

    Returns:

    """
    if request_headers is None:
        request_headers = copy.deepcopy(HEADER_CONTENT_TYPE_APP_JSON)

    request_params_encoded = None
    if request_params:
        request_params_encoded = \
            urllib.parse.urlencode(request_params)

    return command_line_request_curl(
        request_method="GET",
        request_url=request_url,
        request_data=request_params_encoded,
        request_headers=request_headers,
        request_timeout=request_timeout,
        request_allow_redirects=request_allow_redirects
    )


def command_line_request_curl_post(
    request_url,
    request_params,
    request_headers=None,
    request_timeout=60,
    request_data=None,
    request_allow_redirects=True
):
    """Command Line: Build Request cUrl for POST method

    Args:
        request_url:
        request_params:
        request_headers:
        request_timeout:
        request_data:
        request_allow_redirects:

    Returns:

    """
    if request_headers is None:
        request_headers = copy.deepcopy(HEADER_CONTENT_TYPE_APP_JSON)

    if request_params:
        request_url += "?" + urllib.parse.urlencode(request_params)

    return command_line_request_curl(
        request_method="POST",
        request_url=request_url,
        request_headers=request_headers,
        request_data=request_data,
        request_timeout=request_timeout,
        request_allow_redirects=request_allow_redirects
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE MobileAppTracking API base class
"""

import logging

from tune_reporting import (__python_required_version__)
from tune_reporting.support import (python_check_version)
from tune_reporting.tmc.tune_mobileapptracking_api_base import (TuneMobileAppTrackingApiBase)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (TuneV2SessionAuthenticate)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)

python_check_version(__python_required_version__)


# @brief  TUNE MobileAppTracking API base class
#
# @namespace tune_reporting.TuneMobileAppTrackingApi
class TuneMobileAppTrackingApi(TuneMobileAppTrackingApiBase):
    """TUNE MobileAppTracking API base class
    """

    _TUNE_MANAGEMENT_API_ENDPOINT = "https://api.mobileapptracking.com"

    _FILTER_NOT_DEBUG_NOR_TEST_DATA = (
        "(debug_mode = 0 OR debug_mode is NULL)"
        " AND "
        "(test_profile_id = 0 OR test_profile_id IS NULL)"
    )

    #  Advertiser ID
    #  @var str
    __advertiser_id = None

    # Initialize Job
    #
    def __init__(
        self,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR,
        timezone=None
    ):
        """Initialize

        Args:
            integration:
            logger:
        """
        self.session_token = None
        self.advertiser_id = None

        super(TuneMobileAppTrackingApi, self).__init__(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output,
            timezone=timezone
        )

    def get_session_authenticate(self, tmc_api_key, request_retry=None):
        """Generate session token is returned to provide
        access to service.

        Args:
            tmc_api_key:
            request_retry:

        Returns:

        """
        self.logger.info("TMC API: Session Authenticate: Request")

        tmc_auth_response = self.tmc_auth(tmc_api_key=tmc_api_key)
        assert tmc_auth_response

        tune_v2_session_authenticate = TuneV2SessionAuthenticate(integration_name=self.integration_name)

        if tune_v2_session_authenticate.get_session_token(tmc_api_key=tmc_api_key, request_retry=request_retry):
            self.session_token = tune_v2_session_authenticate.session_token
            return True

        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from .constants import (
    HEADER_CONTENT_TYPE_APP_JSON,
    HEADER_CONTENT_TYPE_APP_URLENCODED,
    HEADER_USER_AGENT,
    __MODULE_VERSION_INFO__,
    __MODULE_SIG__,
    __PYTHON_VERSION__,
    __TIMEZONE_NAME_DEFAULT__,
    __USER_AGENT__,
    __LOGGER_NAME__,
)
from .curl import (
    command_line_request_curl,
    command_line_request_curl_get,
    command_line_request_curl_post,
)
from .utils import (
    base_class_name,
    full_class_name,
    convert_size,
    python_check_version,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TMC API Base class
"""

import logging
import requests
import datetime as dt
# from pprintpp import pprint

from pytz_convert import (validate_tz_name)
from pyhttpstatus_utils import (http_status_code_to_type)
from tune_reporting.support import (
    python_check_version,
    __TIMEZONE_NAME_DEFAULT__,
)
from tune_reporting.errors import (
    print_traceback,
    get_exception_message,
    TuneReportingErrorCodes,
)
from requests_mv_integrations.exceptions import (
    TuneRequestBaseError,
    TuneRequestError
)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting import (__version__, __python_required_version__)
from requests_mv_integrations import (RequestMvIntegrationDownload)
from tune_reporting.support import (command_line_request_curl_get)
from logging_mv_integrations import (
    LoggingFormat,
    LoggingOutput,
    get_logger
)
from safe_cast import safe_str

python_check_version(__python_required_version__)


# @brief  TUNE MobileAppTracking API base class
#
# @namespace tune_reporting.TuneMobileAppTrackingApiBase
class TuneMobileAppTrackingApiBase(object):
    """TUNE MobileAppTracking API base class
    """

    _TUNE_MANAGEMENT_API_ENDPOINT = "https://api.mobileapptracking.com"

    __tmc_credentials = None
    __timezone = None
    __mv_request = None
    __logger = None

    @property
    def mv_request(self):
        """Get Property: Logger
        """
        if self.__mv_request is None:
            self.__mv_request = RequestMvIntegrationDownload(
                logger_format=self.logger_format,
                logger_level=self.logger_level,
                logger_output=self.logger_output
            )

        return self.__mv_request

    @property
    def logger(self):
        """Get Property: Logger
        """
        if self.__logger is None:
            self.__logger = get_logger(
                logger_name=__name__.split('.')[0],
                logger_version=__version__,
                logger_format=self.logger_format,
                logger_level=self.logger_level,
                logger_output=self.logger_output
            )

        return self.__logger

    def __init__(
        self,
        logger_level=logging.INFO,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR,
        timezone=None
    ):
        self.logger_level = logger_level
        self.logger_format = logger_format
        self.logger_output = logger_output

        if timezone is not None:
            self.timezone = timezone
        else:
            self.timezone = __TIMEZONE_NAME_DEFAULT__

        self.time_start = dt.datetime.now()

    @property
    def tmc_credentials(self):
        return self.__tmc_credentials

    @tmc_credentials.setter
    def tmc_credentials(self, value):
        self.__tmc_credentials = value

    @property
    def api_key(self):
        return None if self.tmc_credentials is None else self.tmc_credentials.get('api_key', None)

    @api_key.setter
    def api_key(self, value):
        if self.tmc_credentials is None:
            self.tmc_credentials = {}

        if 'api_key' in self.tmc_credentials:
            self.tmc_credentials.pop('api_key')

        self.tmc_credentials.update({'api_key': value})

    @property
    def session_token(self):
        return None if self.tmc_credentials is None \
            else self.tmc_credentials.get('session_token', None)

    @session_token.setter
    def session_token(self, value):
        if self.tmc_credentials is None:
            self.tmc_credentials = {}

        if 'session_token' in self.tmc_credentials:
            self.tmc_credentials.pop('session_token')

        self.tmc_credentials.update({'session_token': value})

    @property
    def timezone(self):
        """Get Property: timezone
        """
        return self.__timezone

    @timezone.setter
    def timezone(self, value):
        """Set Property: timezone
        """
        tz_name = value
        if not validate_tz_name(tz_name):
            raise ValueError("Invalid timezone name assigned: {}".format(tz_name))

        self.__timezone = tz_name

    __data = None

    @property
    def data(self):
        """Provide created reader populated with file data."""
        return self.__data

    @data.setter
    def data(self, value):
        """Provide data value."""
        self.__data = value

    def tune_mat_request_path(self, mat_api_version, controller, action):
        """TUNE Reporting API service path

        Args:
            mat_api_version:
            controller:
            action:

        Returns:

        """
        request_path = "{}/{}/{}/{}".format(self._TUNE_MANAGEMENT_API_ENDPOINT, mat_api_version, controller, action)

        return request_path

    def tune_v2_request_retry_func(self, response):
        """Request Retry Function

        Args:
            response:

        Returns:
            Boolean

        """
        try:
            response_json = response.json()
        except Exception as ex:
            # No JSON response available.
            raise TuneReportingError(
                error_message='Invalid JSON response: {}'.format(response.text),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_JSON_DECODING
            )

        self.logger.debug("TMC API V2: Check for Retry: Start", extra=response_json)

        tune_v2_status_code = None
        tune_v2_errors_messages = ""

        if 'status_code' in response_json:
            tune_v2_status_code = response_json['status_code']

        if 'errors' in response_json:
            tune_v2_errors = response_json['errors']
            if isinstance(tune_v2_errors, dict) \
                    and 'message' in tune_v2_errors:
                tune_v2_errors_messages += tune_v2_errors['message']
            elif isinstance(tune_v2_errors, list):
                for tune_v2_error in tune_v2_errors:
                    if isinstance(tune_v2_error, dict) \
                            and 'message' in tune_v2_error:
                        tune_v2_errors_messages += tune_v2_error['message']

        tune_v2_status_type = http_status_code_to_type(tune_v2_status_code)

        response_extra = {
            'status_code': tune_v2_status_code,
            'status_type': tune_v2_status_type,
        }

        if tune_v2_errors_messages:
            response_extra.update({'error_messages': safe_str(tune_v2_errors_messages)})

        self.logger.debug("TMC API Base: Check for Retry: Response", extra=response_extra)

        if tune_v2_status_code == 200:
            self.logger.debug("TMC API Base: Check for Retry: Success", extra=response_extra)
            return False

        if tune_v2_status_code in [401, 403]:
            self.logger.error("TMC API: Request: Error", extra={'status_code': tune_v2_status_code})
            raise TuneRequestError(
                error_message=tune_v2_errors_messages,
                error_code=tune_v2_status_code,
            )

        if tune_v2_status_code in [404, 500]:
            if "Api key was not found." in tune_v2_errors_messages:
                self.logger.error("TMC API: Request: Error", extra={'tune_v2_status_code': tune_v2_status_code})

                raise TuneRequestError(
                    error_message=tune_v2_errors_messages,
                    error_code=tune_v2_status_code,
                )

            self.logger.warning("TMC API Base: Check for Retry: Retry Candidate", extra=response_extra)
            return True

        self.logger.warning("TMC API Base: Check for Retry: No Retry", extra=response_extra)
        return False

    @property
    def generator(self):
        """Generate report data."""
        if not self.data or len(self.data) == 0:
            yield []
        else:
            for (i, item) in enumerate(self.data):
                yield item

    def tmc_auth(self, tmc_api_key):
        """TMC Authentication"""
        if not tmc_api_key:
            raise ValueError("Parameter 'tmc_api_key' not defined.")

        self.logger.info("TMC Authentication: Start")

        # Attempt to authenticate.
        request_url = ('https://api.mobileapptracking.com/v2/advertiser/find')

        request_params = {'api_key': tmc_api_key}

        auth_request_curl = command_line_request_curl_get(request_url=request_url, request_params=request_params)

        try:
            auth_response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_retry=None,
                request_retry_http_status_codes=None,
                request_retry_func=None,
                request_retry_excps_func=None,
                request_label="TMC Authentication"
            )
        except TuneRequestBaseError as tmc_ex:
            self.logger.error("TMC Authentication: Failed", extra=tmc_ex.to_dict())
            raise

        except Exception as ex:
            print_traceback(ex)
            self.logger.error("TMC Authentication: Failed: {}".format(get_exception_message(ex)))
            raise

        if auth_response.status_code != requests.codes.ok:
            raise TuneReportingError(
                error_message="Invalid request",
                error_request_curl=auth_request_curl,
                error_code=auth_response.status_code
            )

        try:
            decoded_resp = auth_response.json()
        except Exception as ex:
            # No JSON response available.
            raise TuneReportingError(
                error_message='Invalid JSON response: {}'.format(auth_response.text),
                errors=ex,
                error_request_curl=auth_request_curl,
                error_code=TuneReportingErrorCodes.REP_ERR_AUTH_JSON_ERROR
            )

        tmc_status_code = decoded_resp.get('status_code', None)
        tmc_errors = decoded_resp.get('errors', None)

        if tmc_errors:
            error_code = tmc_status_code
            errors = []
            if isinstance(tmc_errors, list):
                error_list = tmc_errors
                error_list = error_list if error_list else []
                for error in error_list:
                    error_message = error.get('message', None)
                    if error_message:
                        errors.append(error_message)
                        if error_message.startswith("Invalid api key"):
                            error_code = TuneReportingErrorCodes.UNAUTHORIZED

            elif isinstance(tmc_errors, dict):
                error_message = tmc_errors.get('message', None)
                if error_message:
                    errors.append(error_message)

            raise TuneReportingError(
                error_message="Error status: {}".format(tmc_errors),
                error_request_curl=auth_request_curl,
                error_code=error_code,
            )

        return decoded_resp



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

import sys

from tune_reporting import (__version__, __title__)

__MODULE_VERSION_INFO__ = tuple(__version__.split('.'))
__MODULE_SIG__ = "%s/%s" % (__title__, __version__)

__TIMEZONE_NAME_DEFAULT__ = "UTC"

__PYTHON_VERSION__ = 'Python/%d.%d.%d' % (sys.version_info[0], sys.version_info[1], sys.version_info[2])

__USER_AGENT__ = "({}, {})".format(__MODULE_SIG__, __PYTHON_VERSION__)

__LOGGER_NAME__ = __name__.split('.')[0]

HEADER_CONTENT_TYPE_APP_JSON = \
    {'Content-Type': 'application/json'}

HEADER_CONTENT_TYPE_APP_URLENCODED = \
    {'Content-Type': 'application/x-www-form-urlencoded'}

HEADER_USER_AGENT = \
    {'User-Agent': __USER_AGENT__}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""
Report Reader Base
"""

from logging import getLogger

from abc import ABCMeta, abstractproperty

log = getLogger(__name__)


# @brief TUNE Multiverse Report Reader abstract base class.
#
# @namespace tune_reporting.ReportReaderBase
class ReportReaderBase(object):
    """Base Abstract class."""

    __metaclass__ = ABCMeta

    __report_url = None

    #  Integration Name
    #  @var str
    __integration = ""

    __data = None

    @property
    def data(self):
        """Provide created reader populated with file data."""
        return self.__data

    @data.setter
    def data(self, value):
        """Provide data value."""
        self.__data = value

    #  The constructor
    #  @param str report_url Download report URL
    #                         of requested report to be exported.
    def __init__(self, report_url):
        """Constructor

            :param str report_url: Download report URL.
        """
        if not report_url:
            raise ValueError(error_message="Undefined 'report_url'")

        if not report_url or \
           not isinstance(report_url, str) or \
           len(report_url) < 1:
            raise ValueError("Parameter 'report_url' is not defined.")

        self.__report_url = report_url

    @abstractproperty
    def read(self):
        """Get property for TuneManagementRequest Action Name."""
        return

    @property
    def report_url(self):
        """REPORT_URL of completed report on SQS."""
        return self.__report_url

    @property
    def generator(self):
        """Generate report data."""
        if not self.data or len(self.data) == 0:
            yield []
        else:
            for (i, item) in enumerate(self.data):
                yield item



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from .management.tmc_v2_advertiser_sites import (TuneV2AdvertiserSites, TuneV2AdvertiserSiteStatus)
from .management.tmc_v2_advertisers import (TuneV2Advertisers)
from .management.tmc_v2_session_authenticate import (TuneV2SessionAuthenticate, TuneV2AuthenticationTypes)
from .reporting.tmc_v2_advertiser_stats_actuals import (TuneV2AdvertiserStatsActuals)
from .reporting.tmc_v2_advertiser_stats_base import (TuneV2AdvertiserStatsActions, TuneV2AdvertiserStatsFormats)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertisers.
"""

import logging

from pyhttpstatus_utils import (HttpStatusType, is_http_status_type)
from requests_mv_integrations.support import (validate_json_response)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (print_traceback, get_exception_message, TuneReportingErrorCodes)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)
from tune_reporting import (__python_required_version__)
from tune_reporting.tmc.tune_mobileapptracking_api_base import (TuneMobileAppTrackingApiBase)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)
# from pprintpp import pprint

python_check_version(__python_required_version__)


# @brief TUNE Advertisers.
#
# @namespace tune_reporting.TuneV2AdvertiserStatus
class TuneV2AdvertiserStatus(object):
    """Status states for Advertisers.
    """
    ACTIVE = 1
    NOT_ACTIVE = 2
    ALL = 3


#   TUNE MobileAppTracking: Advertisers
#
class TuneV2Advertisers(TuneMobileAppTrackingApiBase):
    """TUNE Advertisers.
    """

    #  Advertiser ID
    #  @var str
    __advertiser_id = None

    # Initialize Job
    #
    def __init__(
        self,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR
    ):
        super(TuneV2Advertisers, self).__init__(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

    @property
    def advertiser_id(self):
        """Get Property: advertiser_id
        """
        return self.__advertiser_id

    @advertiser_id.setter
    def advertiser_id(self, value):
        """Set Property: advertiser_id
        """
        self.__advertiser_id = value

    # Collect Advertisers
    #
    def get_advertiser_id(self, auth_type, auth_value, request_retry=None):
        """Get Advertiser ID

        Args:
            auth_type:
            auth_value:
            request_retry:

        Returns:

        """
        if not auth_type:
            raise ValueError("TMC v2 Advertisers: Get Advertiser ID: Value 'auth_type' not provided.")
        if not auth_value:
            raise ValueError("TMC v2 Advertisers: Get Advertiser ID: Value 'auth_value' not provided.")

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller="advertiser",
                action="find"
            )

        request_params = {auth_type: auth_value}

        self.logger.info("TMC v2 Advertisers: Advertiser ID")

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_retry=None,
                request_retry_http_status_codes=None,
                request_retry_func=self.tune_v2_request_retry_func,
                request_retry_excps_func=None,
                request_label="TMC v2 Advertisers"
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v2 Advertisers: Advertiser ID: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v2 Advertisers: Advertiser ID: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error(
                "TMC v2 Advertisers: Advertiser ID: Failed",
                extra={'error': get_exception_message(ex)},
            )

            raise TuneReportingError(
                error_message=("TMC v2 Advertisers: Failed: {}").format(get_exception_message(ex)),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v2 Advertisers: Advertiser ID:"
        )

        self.logger.debug("TMC v2 Advertisers: Advertiser ID", extra={'response': json_response})

        json_response_status_code = json_response['status_code']

        http_status_successful = is_http_status_type(
            http_status_code=json_response_status_code, http_status_type=HttpStatusType.SUCCESSFUL
        )

        if not http_status_successful or not json_response['data']:
            raise TuneReportingError(
                error_message="TMC v2 Advertisers: Failed: {}".format(json_response_status_code),
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        if 'data' not in json_response or \
                not json_response['data'] or \
                len(json_response['data']) == 0:
            raise TuneReportingError(
                error_message="TMC v2 Advertisers: Advertiser ID: Failed",
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        advertiser_data = json_response['data'][0]

        self.advertiser_id = advertiser_data.get('id', None)
        if self.advertiser_id is None:
            raise TuneReportingError(
                error_message="TMC v2 Advertisers: Advertiser ID: Failed",
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        self.logger.info("TMC v2 Advertisers: Advertiser ID: {}".format(self.advertiser_id))

        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from .report_reader_base import (ReportReaderBase)
from .report_reader_csv import (ReportReaderCSV)
from .report_reader_json import (ReportReaderJSON)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting.errors

from requests_mv_integrations.errors import (TuneRequestErrorCodes)


class TuneReportingErrorCodes(TuneRequestErrorCodes):
    """TUNE Reporting Error Codes
    """

    REP_ERR_UNASSIGNED = -1

    MOD_OK = 0  # Success

    #
    # 7xx Tune Reporting Errors
    #
    REP_ERR_REPORTING = 700  # Reporting Error

    REP_ERR_REQUEST = 707
    # Exit code that request failed.

    REP_ERR_SOFTWARE = 708
    # Exit code that means an internal
    # software error was detected.

    REP_ERR_UNEXPECTED_VALUE = 710
    # Unexpected value either
    # returned or null.

    REP_ERR_JOB_STOPPED = 721  # Job Stopped
    REP_ERR_RETRY_EXHAUSTED = 722  # Retry Exhausted

    REP_ERR_AUTH_ERROR = 760  # Auth Error
    REP_ERR_AUTH_JSON_ERROR = 761  # Auth JSON Error

    REP_ERR_JSON_DECODING = 766  # JSON Decoding Error

    REP_ERR_UNEXPECTED = 799  # Unexpected Error



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from .base import (TuneReportingBaseError)
from .custom import (TuneReportingError, TuneReportingAuthError)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""
TUNE Multiverse Reporting Base
================================
"""

import logging

from logging_mv_integrations import (LoggingFormat, LoggingOutput)

from tune_reporting.errors import (
    get_exception_message,
    print_traceback,
)
from tune_reporting.exceptions import (TuneReportingAuthError, TuneReportingError)
from tune_reporting.support import (base_class_name)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (TuneV2SessionAuthenticate)

log = logging.getLogger(__name__)


def tmc_auth_v2_session_token(
    tmc_api_key,
    logger_level=logging.NOTSET,
    logger_format=LoggingFormat.JSON,
    logger_output=LoggingOutput.STDOUT_COLOR
):
    """TMC Authentication

    :return:
    """
    log.info("TMC v2 Session Token: Request")

    try:
        tune_v2_session_authenticate = \
            TuneV2SessionAuthenticate(
                logger_level=logger_level,
                logger_format=logger_format,
                logger_output=logger_output
            )

        if tune_v2_session_authenticate.get_session_token(tmc_api_key=tmc_api_key, request_retry=None):
            session_token = tune_v2_session_authenticate.session_token

    except TuneReportingError as auth_ex:
        log.error("TMC v2 Session Token: Failed", extra=auth_ex.to_dict())

        raise

    except Exception as ex:
        print_traceback(ex)
        log.error(
            'TMC v2 Session Token: Failed: Unexpected',
            extra={'error_exception': base_class_name(ex),
                   'error_details': get_exception_message(ex)}
        )

        raise TuneReportingAuthError(
            error_message="TMC v2 Session Token: Failed: Unexpected",
            errors=ex,
        )

    log.debug("TMC v2 Session Token: Response", extra={'session_token': session_token})

    return session_token



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertiser Stats Abstract Base Class.
"""

import logging
import datetime as dt
import ujson as json
import os
import time
from abc import ABCMeta, abstractmethod
from urllib.parse import urlparse

from pyhttpstatus_utils import (
    HttpStatusType,
    HttpStatusCode,
    is_http_status_type,
)
from requests_mv_integrations.support import (validate_json_response)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (print_traceback, get_exception_message, TuneReportingErrorCodes)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)
from tune_reporting.readers.report_reader_json import (ReportReaderJSON)
from tune_reporting import (__python_required_version__)
from tune_reporting.readers.report_reader_csv import (ReportReaderCSV)
from tune_reporting.tmc.v2.management import (TuneV2AuthenticationTypes)
from tune_reporting.tmc.tmc_auth_v2_session_token import (tmc_auth_v2_session_token)
from tune_reporting.tmc.tune_mobileapptracking_api import (TuneMobileAppTrackingApi)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)
from safe_cast import (
    safe_dict,
    safe_int,
)
python_check_version(__python_required_version__)


# @brief TUNE Advertiser Stats Actions ENUM
#
# @namespace tune_reporting.TuneV2AdvertiserStatsExportActions
class TuneV2AdvertiserStatsActions(object):
    """Status states for Advertiser Sites.
    """
    FIND = "find"
    EXPORT = "export"


# @brief TUNE Advertiser Stats Export Actions ENUM
#
# @namespace tune_reporting.TuneV2AdvertiserStatsExportActions
class TuneV2AdvertiserStatsExportActions(object):
    """Export actions for Advertiser Sites.
    """
    EXPORT = "export"
    FIND_EXPORT_QUEUE = "find_export_queue"


# @brief TUNE Advertiser Stats Status Actions ENUM
#
# @namespace tune_reporting.TuneV2AdvertiserStatsStatusAction
class TuneV2AdvertiserStatsStatusAction(object):
    """Export Status actions for Advertiser Sites.
    """
    STATUS = "status"
    DOWNLOAD = "download"


# @brief TUNE Advertiser Stats Formats ENUM
#
# @namespace tune_reporting.TuneV2AdvertiserStatsFormats
class TuneV2AdvertiserStatsFormats(object):
    """Data formats for Advertiser Sites.
    """
    JSON = "json"
    CSV = "csv"


# @brief TUNE Advertiser Stats Abstract Base Class.
#
# @namespace tune_reporting.TuneV2AdvertiserStatsBase
class TuneV2AdvertiserStatsBase(TuneMobileAppTrackingApi):
    """TUNE Advertiser Stats Abstract Base Class.
    """

    __metaclass__ = ABCMeta

    _FILTER_NOT_DEBUG_NOR_TEST_DATA = (
        "(debug_mode = 0 OR debug_mode is NULL)"
        " AND "
        "(test_profile_id = 0 OR test_profile_id IS NULL)"
    )

    _TUNE_ADVERTISING_STATS_TPL = \
        "tmc_advertising_stats_{job_id}.{format}"

    # Initialize Job
    #
    def __init__(
        self,
        controller,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR,
        timezone=None
    ):
        """Initialize TUNE Advertiser Stats Abstract Base Class.
        """
        super(TuneV2AdvertiserStatsBase, self).__init__(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output,
            timezone=timezone
        )

        self.controller = controller

    # Collect data: TUNE Advertiser Stats Actuals.
    #
    def collect(
        self,
        auth_value,
        auth_type,
        auth_type_use,
        start_date,
        end_date,
        request_params=None,
        request_retry=None,
        request_action=TuneV2AdvertiserStatsActions.FIND
    ):
        """Collect data: TUNE Advertiser Stats Base

        Args:
            auth_value:
            auth_type:
            start_date:
            end_date:
            request_params:
            request_retry:
            request_action:

        Returns:

        """
        self.logger.info("TMC v2 Advertiser Stats: Collect: {}".format(request_action))

        if not auth_value:
            raise ValueError(error_message="TMC v2 Advertiser Stats: Collect: Value 'auth_value' not provided.")
        if not auth_type or \
                not TuneV2AuthenticationTypes.validate(auth_type):
            raise ValueError(error_message="TMC v2 Advertiser Stats: Collect: Value 'auth_type' not valid.")
        if not auth_type_use or \
                not TuneV2AuthenticationTypes.validate(auth_type_use):
            raise ValueError(error_message="TMC v2 Advertiser Stats: Collect: Value 'auth_type_use' not valid.")

        if auth_type == TuneV2AuthenticationTypes.API_KEY:
            self.api_key = auth_value

            if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
                self.session_token = tmc_auth_v2_session_token(
                    tmc_api_key=self.api_key, logger_level=self.logger_level, logger_format=self.logger_format
                )
        elif auth_type == TuneV2AuthenticationTypes.SESSION_TOKEN:
            self.session_token = auth_value
        else:
            raise ValueError(error_message="Invalid 'auth_type': '{}'".format(auth_type))

        auth_value_use = None
        if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            if self.session_token is None:
                raise TuneReportingError(
                    error_message="Value 'session_token' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
                )

            auth_value_use = self.session_token

        elif auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            if self.api_key is None:
                raise TuneReportingError(
                    error_message="Value 'api_key' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE,
                )

            auth_value_use = self.api_key

        if not auth_value_use:
            raise TuneReportingError(
                error_message="Value 'auth_value_use' not defined.",
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        self._collect(auth_type_use, start_date, end_date, request_params, request_retry, request_action)

    # Stream data: TUNE Advertiser Stats Base
    #
    def stream(
        self,
        auth_value,
        auth_type,
        auth_type_use,
        start_date,
        end_date,
        request_params,
        request_retry=None,
    ):
        """Stream data: TUNE Advertiser Stats Base

        Args:
            auth_value:
            auth_type:
            start_date:
            end_date:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.info("TMC v2 Advertiser Stats: Stream")

        if not auth_value:
            raise ValueError(error_message="TMC v2 Advertiser Stats: Stream: Value 'auth_value' not provided.")
        if not auth_type or \
                not TuneV2AuthenticationTypes.validate(auth_type):
            raise ValueError(error_message="TMC v2 Advertiser Stats: Stream: Value 'auth_type' not valid.")
        if not auth_type_use or \
                not TuneV2AuthenticationTypes.validate(auth_type_use):
            raise ValueError(error_message="TMC v2 Advertiser Stats: Stream: Value 'auth_type_use' not valid.")

        if not request_params:
            raise ValueError(error_message="Value 'request_params' not provided.")

        if auth_type == TuneV2AuthenticationTypes.API_KEY:
            self.api_key = auth_value

            if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
                self.session_token = tmc_auth_v2_session_token(
                    tmc_api_key=self.api_key, logger_level=self.logger_level, logger_format=self.logger_format
                )
        elif auth_type == TuneV2AuthenticationTypes.SESSION_TOKEN:
            self.session_token = auth_value
        else:
            raise ValueError(error_message="Invalid 'auth_type': '{}'".format(auth_type))

        auth_value_use = None
        if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            if self.session_token is None:
                raise TuneReportingError(
                    error_message="Value 'session_token' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
                )

            auth_value_use = self.session_token

        elif auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            if self.api_key is None:
                raise TuneReportingError(
                    error_message="Value 'api_key' not defined.",
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE,
                )

            auth_value_use = self.api_key

        if not auth_value_use:
            raise TuneReportingError(
                error_message="Value 'auth_value_use' not defined.",
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        return self._stream_v2(
            auth_type_use=auth_type_use,
            start_date=start_date,
            end_date=end_date,
            request_params=request_params,
            request_retry=request_retry
        )

    @abstractmethod
    def _collect(
        self,
        auth_type_use,
        start_date,
        end_date,
        request_params=None,
        request_retry=None,
        request_action=TuneV2AdvertiserStatsActions.FIND,
    ):
        """Collect data: TUNE Advertiser Stats.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:
            request_action:

        Returns:

        """

        raise NotImplementedError

    @abstractmethod
    def _stream_v2(
        self,
        auth_type_use,
        start_date,
        end_date,
        request_params,
        request_retry=None,
    ):
        """Collect data: TUNE Advertiser Stats.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:

        Returns:

        """

        raise NotImplementedError

    def _find_v2(
        self,
        request_params,
        request_retry=None,
        request_label="TMC v2 Advertiser Stats Find",
    ):
        """Gather data using action find.json.
        """

        self.logger.debug("TuneV2AdvertiserStatsBase", extra={'action': 'find'})

        if "start_date" not in request_params:
            raise ValueError("Missing attribute 'start_date' in parameter 'request_params'")
        request_start_date = request_params["start_date"]

        if "end_date" not in request_params:
            raise ValueError("Missing attribute 'end_date' in parameter 'request_params'")
        request_end_date = request_params["end_date"]

        datetime_start = dt.datetime.strptime(request_start_date, '%Y-%m-%d')
        datetime_end = dt.datetime.strptime(request_end_date, '%Y-%m-%d')

        str_date_start = str(datetime_start.date())
        str_date_end = str(datetime_end.date())

        self.logger.debug(("TuneV2AdvertiserStatsBase"),
                          extra={'action': 'find',
                                 'start_date': str_date_start,
                                 'end_date': str_date_end})

        if not request_retry:
            request_retry = {'timeout': 60}

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller=self.controller,
                action="find"
            )

        if 'format' not in request_params:
            request_params['format'] = TuneV2AdvertiserStatsFormats.JSON

        request_params["start_date"] += " 00:00:00"
        request_params["end_date"] += " 23:59:59"

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_label=request_label
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v2 Advertiser Stats: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v2 Advertiser Stats: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TMC v2 Advertiser Stats: {}".format(get_exception_message(ex)))
            raise

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v2 Advertiser Stats: Action 'find'"
        )

        if json_response['status_code'] != 200:
            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: "
                               "Action 'find': "
                               "Failed to find stats: {}, {}").format(
                                   json_response['status_code'],
                                   json.dumps(json_response),
                               ),
                error_code=TuneReportingErrorCodes.REP_ERR_REQUEST
            )

        data = json_response['data']
        self.data = data

    def _export_v2_download_csv(
        self,
        tmp_directory,
        auth_type_use,
        export_controller,
        export_action,
        export_status_controller,
        export_status_action,
        request_params,
        request_retry=None,
    ):
        """Gather Export by Downloading CSV and reading Data

        Args:
            export_controller:
            export_action:
            export_status_controller:
            export_status_action:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.debug(("TMC v2 Advertiser Stats: Export CSV Download: "
                           "Actions '{}' and '{}'").format(export_action, export_status_action))

        if not request_params:
            raise ValueError("Missing parameter 'request_params'")

        if "start_date" not in request_params:
            raise ValueError("Missing attribute 'start_date' in parameter 'request_params'")
        request_start_date = request_params["start_date"]

        if "end_date" not in request_params:
            raise ValueError("Missing attribute 'end_date' in parameter 'request_params'")
        request_end_date = request_params["end_date"]

        if 'format' not in request_params:
            request_params['format'] = TuneV2AdvertiserStatsFormats.CSV
        export_format = request_params['format']

        datetime_start = dt.datetime.strptime(request_start_date, '%Y-%m-%d')
        datetime_end = dt.datetime.strptime(request_end_date, '%Y-%m-%d')

        str_date_start = str(datetime_start.date())
        str_date_end = str(datetime_end.date())

        self.logger.debug(("TMC v2 Advertiser Stats: Export CSV Download"),
                          extra={
                              'action': 'export',
                              'start_date': str_date_start,
                              'end_date': str_date_end,
                              'export_format': export_format
                          })

        data = []

        data, job_row_count = self._process_export_download_csv_v2(
            tmp_directory,
            data,
            auth_type_use,
            str_date_start,
            str_date_end,
            export_controller,
            export_action,
            export_status_controller,
            export_status_action,
            request_params,
            request_retry,
        )

        self.data = data

    def _export_stream_v2(
        self,
        auth_type_use,
        export_controller,
        export_action,
        export_status_controller,
        export_status_action,
        request_params,
        request_retry=None,
    ):
        """Gather Export using Requests Stream

        Args:
            export_controller:
            export_action:
            export_status_controller:
            export_status_action:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.debug(
            "TMC v2 Advertiser Stats: Gather Export by Requests Stream",
            extra={
                'export_action': export_action,
                'export_status_action': export_status_action,
                'request_retry': request_retry
            }
        )

        if not request_params:
            raise ValueError("Missing parameter 'request_params'")

        if "start_date" not in request_params:
            raise ValueError("Missing attribute 'start_date' in parameter 'request_params'")
        request_start_date = request_params["start_date"]

        if "end_date" not in request_params:
            raise ValueError("Missing attribute 'end_date' in parameter 'request_params'")
        request_end_date = request_params["end_date"]

        if 'format' not in request_params:
            request_params['format'] = TuneV2AdvertiserStatsFormats.CSV
        export_format = request_params['format']

        datetime_start = dt.datetime.strptime(request_start_date, '%Y-%m-%d')
        datetime_end = dt.datetime.strptime(request_end_date, '%Y-%m-%d')

        str_date_start = str(datetime_start.date())
        str_date_end = str(datetime_end.date())

        self.logger.debug(
            "TMC v2 Advertiser Stats: Gather Export by Requests Stream",
            extra={
                'action': 'export',
                'start_date': str_date_start,
                'end_date': str_date_end,
                'export_format': export_format,
                'request_retry': request_retry
            }
        )

        return self._process_export_stream_v2(
            auth_type_use,
            str_date_start,
            str_date_end,
            export_controller,
            export_action,
            export_status_controller,
            export_status_action,
            request_params,
            request_retry,
        )

    def _process_export_download_csv_v2(
        self,
        tmp_directory,
        data,
        auth_type_use,
        str_date_start,
        str_date_end,
        export_controller,
        export_action,
        export_status_controller,
        export_status_action,
        request_params,
        request_retry,
    ):
        """Process Export Job by Reading Downloaded CSV

        Args:
            data:
            str_date_start:
            str_date_end:
            export_controller:
            export_action:
            export_status_controller:
            export_status_action:
            request_params:
            request_retry:

        Returns:

        """
        auth_value = None
        if auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            auth_value = self.api_key
        elif auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            auth_value = self.session_token

        str_date_start += " 00:00:00"
        str_date_end += " 23:59:59"

        request_params["start_date"] = \
            str_date_start
        request_params["end_date"] = \
            str_date_end

        export_job_id = self._export_v2_job_to_queue(
            export_controller,
            export_action,
            request_params,
            request_retry,
        )

        export_report_url = self._check_v2_job_status_on_queue(
            auth_type_use,
            auth_value,
            export_status_controller,
            export_status_action,
            export_job_id,
            request_retry=request_retry,
        )

        if not export_report_url:
            raise TuneReportingError(
                error_message="Export URL not defined",
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE,
            )

        self.logger.info("TuneV2AdvertiserStatsBase", extra={'job_id': export_job_id, 'report_url': export_report_url})

        tmp_csv_file_name = self._TUNE_ADVERTISING_STATS_TPL.format(
            job_id=export_job_id, format=TuneV2AdvertiserStatsFormats.CSV
        )

        tmp_csv_file_name = tmp_csv_file_name.replace('-', '_')

        job_row_count = 0

        for row in self.mv_request.request_csv_download(
            tmp_csv_file_name=tmp_csv_file_name,
            tmp_directory=tmp_directory,
            request_method="GET",
            request_url=export_report_url,
            request_label="TMC v2 Advertiser Stats: Export: Download CSV"
        ):
            # for row in list(generator_data_stats):
            if len(row) == 0:
                continue

            job_row_count += 1
            data.append(row)

        if os.path.exists(tmp_csv_file_name):
            os.remove(tmp_csv_file_name)

        self.logger.debug(
            "TMC v2 Advertiser Stats: Process Export Job by Reading Downloaded CSV",
            extra={
                'action': 'export',
                'start_date': str_date_start,
                'end_date': str_date_end,
                'job_id': export_job_id,
                'row_count': job_row_count
            }
        )

        return (data, job_row_count)

    def _process_export_stream_v2(
        self,
        auth_type_use,
        str_date_start,
        str_date_end,
        export_controller,
        export_action,
        export_status_controller,
        export_status_action,
        request_params,
        request_retry,
        request_label="TMC v2 Advertiser Stats Export Stream",
    ):
        """Process Export Job by Steaming

        Args:
            str_date_start:
            str_date_end:
            export_controller:
            export_action:
            export_status_controller:
            export_status_action:
            request_params:
            request_retry:

        Returns:

        """
        auth_value = None
        if auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            auth_value = self.api_key
        elif auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            auth_value = self.session_token

        str_date_start += " 00:00:00"
        str_date_end += " 23:59:59"

        request_params["start_date"] = \
            str_date_start
        request_params["end_date"] = \
            str_date_end

        self.logger.debug(
            "TMC v2 Advertiser Stats: Export Stream V2: Export Job to Queue",
            extra={
                'controller': export_controller,
                'action': export_action,
                'request_params': safe_dict(request_params),
                'request_retry': safe_dict(request_retry)
            }
        )

        export_job_id = self._export_v2_job_to_queue(
            export_controller,
            export_action,
            request_params,
            request_retry,
        )

        self.logger.debug(
            "TMC v2 Advertiser Stats: Export Stream V2: Check Job status on Queue",
            extra={'controller': export_status_controller,
                   'action': export_status_action,
                   'job_id': export_job_id}
        )

        export_report_url = self._check_v2_job_status_on_queue(
            auth_type_use,
            auth_value,
            export_status_controller,
            export_status_action,
            export_job_id,
            request_retry=request_retry
        )

        if not export_report_url:
            raise TuneReportingError(
                error_message="Export URL not defined",
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE,
            )

        self.logger.info(("TMC v2 Advertiser Stats: "
                          "Export Stream V2: Request Completed Job"),
                         extra={'job_id': export_job_id,
                                'report_url': export_report_url})

        response = self.mv_request.request(
            request_method="GET", request_url=export_report_url, stream=True, request_label=request_label
        )

        self.logger.info(
            "TMC v2 Advertiser Stats: Export Stream V2: Response Completed Job",
            extra={
                'response_status_code': response.status_code,
                'response_headers': response.headers,
                'job_id': export_job_id,
                'report_url': export_report_url
            }
        )

        return response

    def _export_v2_job_to_queue(
        self,
        export_controller,
        export_action,
        request_params,
        request_retry=None,
        request_label="TMC v2 Job To Queue",
    ):
        """Export Report Request to Job Queue

        Args:
            export_controller:
            export_action:
            request_params:
            request_retry:

        Returns:
            Export Job ID

        """
        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller=export_controller,
                action=export_action
            )

        self.logger.debug(("TMC v2 Advertiser Stats: "
                           "Start Advertiser Report Actuals"),
                          extra={
                              'action': export_action,
                              'start_date': request_params["start_date"],
                              'end_date': request_params["end_date"]
                          })

        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=request_params,
                request_label=request_label,
                request_retry_func=self.tune_v2_request_retry_func
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v2 Advertiser Stats: Request Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v2 Advertiser Stats: Reporting Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error(get_exception_message(ex))

            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: Unexpected Failed: {}").format(get_exception_message(ex)),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        json_response = validate_json_response(
            response,
            request_curl=self.mv_request.built_request_curl,
            request_label="TMC v2 Advertiser Stats: Action '{}'".format(export_action)
        )

        if hasattr(response, 'url'):
            self.logger.info("TMC v2 Advertiser Stats: Reporting API: Export URL", extra={'response_url': response.url})

        if (not json_response or json_response['status_code'] != 200 or 'errors' in json_response):
            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: "
                               "Action '{}': "
                               "Failed to export stats: {}, {}").format(
                                   export_action, json_response['status_code'], json_response['errors']
                               ),
                error_code=TuneReportingErrorCodes.REP_ERR_REQUEST
            )

        if ('data' not in json_response or not json_response['data']):
            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: "
                               "Action '{}': "
                               "Missing data").format(export_action),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )
        json_data = json_response['data']

        export_job_id = None
        if export_action == TuneV2AdvertiserStatsExportActions.EXPORT:
            if ('job_id' not in json_data or not json_data['job_id']):
                raise TuneReportingError(
                    error_message=("TMC v2 Advertiser Stats: "
                                   "Action '{}': "
                                   "Response missing 'export_job_id': {}").format(export_action, str(json_data)),
                    error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
                )

            export_job_id = json_data['job_id']
        elif export_action == TuneV2AdvertiserStatsExportActions.FIND_EXPORT_QUEUE:
            export_job_id = json_data

        if not export_job_id:
            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: "
                               "Action '{}': "
                               "Response missing 'job_id'").format(export_action),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )

        self.logger.info("TMC v2 Advertiser Stats: Reporting API: Job ID", extra={'job_id': export_job_id})

        return export_job_id

    def _check_v2_job_status_on_queue(
        self,
        auth_type,
        auth_value,
        export_status_controller,
        export_status_action,
        export_job_id,
        request_retry=None,
    ):
        """Check Job Export Status

        Args:
            export_status_controller:
            export_status_action:
            export_job_id:
            request_retry:

        Returns:

        """
        request_label = "TMC v2 Advertiser Stats: Check Export Status"

        v2_export_status_request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller=export_status_controller,
                action=export_status_action
            )

        request_params = {auth_type: auth_value, "job_id": export_job_id}

        self.logger.info(("TMC v2 Advertiser Stats: Check Job Status"),
                         extra={
                             'action': export_status_action,
                             'job_id': export_job_id,
                             'request_url': v2_export_status_request_url,
                             'request_params': safe_dict(request_params)
                         })

        tries = 60  # -1 (indefinite)
        delay = 10
        jitter = 10
        max_delay = 60

        if request_retry is not None:
            if 'delay' in request_retry:
                delay = request_retry['delay']
            if 'jitter' in request_retry:
                jitter = request_retry['jitter']
            if 'max_delay' in request_retry:
                max_delay = request_retry['max_delay']

            if 'tries' in request_retry:
                tries = request_retry['tries']
            else:
                request_retry.update({'tries': 60})
        else:
            request_retry = {'tries': 60, 'delay': 10, 'timeout': 60}

        self.logger.debug(msg=("TMC v2 Advertiser Stats: Check Job Status: " "Request Retry"), extra=request_retry)

        report_url = None
        _attempts = 1
        export_percent_complete = 0

        time.sleep(10)

        _tries, _delay = tries, delay
        while True:
            try:
                response = self.mv_request.request(
                    request_method="GET",
                    request_url=v2_export_status_request_url,
                    request_params=request_params,
                    request_label=request_label,
                    request_retry_func=self.tune_v2_request_retry_func
                )

            except TuneRequestBaseError as tmc_req_ex:
                self.logger.error(
                    "TMC v2 Advertiser Stats: Check Job Status: Failed",
                    extra=tmc_req_ex.to_dict(),
                )
                raise

            except TuneReportingError as tmc_rep_ex:
                self.logger.error(
                    "TMC v2 Advertiser Stats: Check Job Status: Failed",
                    extra=tmc_rep_ex.to_dict(),
                )
                raise

            except Exception as ex:
                print_traceback(ex)

                self.logger.error("TMC v2 Advertiser Stats: Check Job Status: {}".format(get_exception_message(ex)))
                raise

            http_status_successful = is_http_status_type(
                http_status_code=response.status_code, http_status_type=HttpStatusType.SUCCESSFUL
            )

            if not http_status_successful:
                raise TuneReportingError(
                    error_message=("Failed to get export status on queue: {}").format(response.status_code),
                    error_code=TuneReportingErrorCodes.REP_ERR_REQUEST
                )

            if hasattr(response, 'url'):
                self.logger.info(
                    "TMC v2 Advertiser Stats: Reporting API: Status URL", extra={'response_url': response.url}
                )

            json_response = response.json()

            if not json_response:
                request_status_successful = False

            elif 'status_code' not in json_response:
                request_status_successful = False

            else:
                status_code = json_response['status_code']

                request_status_successful = is_http_status_type(
                    http_status_code=status_code, http_status_type=HttpStatusType.SUCCESSFUL
                )

                errors = None
                if 'errors' in json_response:
                    errors = json_response['errors']

            if not request_status_successful:
                error_message = ("TMC v2 Advertiser Stats: Check Job Status: GET '{}', Failed: {}, {}").format(
                    v2_export_status_request_url, status_code, errors
                )

                if (status_code == TuneReportingError.EX_SRV_ERR_500_INTERNAL_SERVER):
                    self.logger.error(error_message)

                elif (status_code == TuneReportingError.EX_SRV_ERR_503_SERVICE_UNAVAILABLE):
                    self.logger.error(error_message)

                elif (status_code == TuneReportingError.EX_SRV_ERR_504_SERVICE_TIMEOUT):
                    self.logger.error(error_message)
                    continue

                elif (status_code == TuneReportingError.EX_CLT_ERR_408_REQUEST_TIMEOUT):
                    self.logger.error(
                        "GET '{}' request timeout, Retrying: {}".format(v2_export_status_request_url, status_code)
                    )
                    continue

                else:
                    raise TuneReportingError(error_message=error_message, error_code=status_code)

                if tries >= 0 and _tries <= 1:
                    if (status_code == HttpStatusCode.GATEWAY_TIMEOUT):
                        raise TuneReportingError(
                            error_message=error_message, error_code=TuneReportingErrorCodes.GATEWAY_TIMEOUT
                        )
                    elif (status_code == HttpStatusCode.REQUEST_TIMEOUT):
                        raise TuneReportingError(
                            error_message=error_message, error_code=TuneReportingErrorCodes.REQUEST_TIMEOUT
                        )
                    else:
                        raise TuneReportingError(error_message=error_message, error_code=status_code)
                else:
                    self.logger.warning(error_message)

            export_percent_complete = 0
            if 'data' in json_response and json_response['data']:
                json_data = json_response['data']

                if "percent_complete" in json_data:
                    export_percent_complete = \
                        safe_int(json_data["percent_complete"])

                self.logger.info(
                    msg=("TMC v2 Advertiser Stats: "
                         "Check Job Export Status: "
                         "Response Success"),
                    extra={
                        'job_id': export_job_id,
                        'export_status': json_data["status"],
                        'export_percent_complete': safe_int(export_percent_complete),
                        'attempt': _attempts
                    }
                )

                if (export_status_action == TuneV2AdvertiserStatsStatusAction.STATUS):
                    if (export_percent_complete == 100 and json_data["status"] == "complete" and json_data["url"]):
                        report_url = json_data["url"]

                        self.logger.debug(("TMC v2 Advertiser Stats: "
                                           "Check Job Export Status: Completed"),
                                          extra={
                                              'job_id': export_job_id,
                                              'action': export_status_action,
                                              'report_url': report_url,
                                              'request_label': request_label
                                          })

                        break

                elif (export_status_action == TuneV2AdvertiserStatsStatusAction.DOWNLOAD):
                    if (export_percent_complete == 100 and
                            json_data["status"] == "complete" and
                            json_data["data"]["url"]):
                        report_url = json_data["data"]["url"]

                        self.logger.debug(("TMC v2 Advertiser Stats: "
                                           "Check Job Export Status: Completed"),
                                          extra={
                                              'job_id': export_job_id,
                                              'action': export_status_action,
                                              'report_url': report_url,
                                              'request_label': request_label
                                          })

                        break
            else:
                self.logger.debug("TMC v2 Advertiser Stats: " "Check Job Export Status: " "No Data Available")

            if tries >= 0:
                _tries -= 1
                if _tries == 0:
                    self.logger.error(("TMC v2 Advertiser Stats: "
                                       "Check Job Export Status: Exhausted Retries"),
                                      extra={
                                          'attempt': _attempts,
                                          'tries': _tries,
                                          'action': export_status_action,
                                          'request_label': request_label,
                                          'export_percent_complete': safe_int(export_percent_complete),
                                          'job_id': export_job_id
                                      })

                    raise TuneReportingError(
                        error_message=(
                            "TMC v2 Advertiser Stats: "
                            "Check Job Export Status: "
                            "Exhausted Retries: "
                            "Percent Completed: {}"
                        ).format(safe_int(export_percent_complete)),
                        error_code=TuneReportingErrorCodes.REP_ERR_RETRY_EXHAUSTED
                    )

            _attempts += 1

            self.logger.info(
                "TMC v2 Advertiser Stats: Check Job Status",
                extra={'attempt': _attempts,
                       'job_id': export_job_id,
                       'delay': _delay,
                       'action': export_status_action}
            )

            time.sleep(_delay)

            _delay += jitter
            _delay = min(_delay, max_delay)

        if export_percent_complete == 100 and not report_url:
            raise TuneReportingError(
                error_message=("TMC v2 Advertiser Stats: Check Job Export Status: "
                               "Download report URL: Undefined"),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )

        self.logger.info(
            "TMC v2 Advertiser Stats: Check Job Export Status: Finished",
            extra={
                'attempt': _attempts,
                'action': export_status_action,
                'report_url': report_url,
                'request_label': request_label,
                'export_percent_complete': export_percent_complete,
                'job_id': export_job_id
            }
        )

        return report_url

    def _read_stats_data(self, export_job_id, report_url, report_format=None):
        """Read v2 Advertiser Stats Data

        Args:
            export_job_id:
            report_url:
            report_format:

        Returns:

        """
        if not export_job_id:
            raise ValueError("Argument 'job_id' not assigned.")
        if not report_url:
            raise ValueError("Argument 'report_url' not assigned.")

        self.logger.info(
            msg=("TMC v2 Advertiser Stats: "
                 "Start Advertiser Report Stats Read"), extra={'job_id': export_job_id}
        )

        if not report_format:
            path = urlparse(report_url).path
            ext = os.path.splitext(path)[1]
            report_format = ext[1:]

        self.logger.info(
            "TMC v2 Advertiser Stats: Reading", extra={'report_url': report_url,
                                                       'report_format': report_format}
        )

        # pylint: disable=redefined-variable-type
        if report_format == TuneV2AdvertiserStatsFormats.JSON:
            report_reader = ReportReaderJSON(report_url)
        elif report_format == TuneV2AdvertiserStatsFormats.CSV:
            report_reader = ReportReaderCSV(report_url)
        else:
            raise TuneReportingError(
                error_message=("Unexpected Report format: '{}'").format(report_format),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )
        # pylint: enable=redefined-variable-type

        if not report_reader:
            raise TuneReportingError(
                error_message=("Report reader not created for format {}").format(report_format),
                error_code=TuneReportingErrorCodes.REP_ERR_UNEXPECTED_VALUE
            )

        report_reader.read()

        self.logger.info(("TMC v2 Advertiser Stats: " "Finished Advertiser Report Stats read"))

        return report_reader.generator



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting.errors

from requests_mv_integrations.errors import error_name as tune_requests_error_name
from requests_mv_integrations.errors import error_desc as tune_requests_error_desc

tune_reporting_error_name = {
    700: 'Reporting Error',
    707: 'Request Error',
    708: 'Software Error',
    710: 'Unexpected Value',
    721: 'Job Stopped',
    722: 'Retry Exhausted',
    760: 'Auth Error',
    761: 'Auth JSON Error',
    766: 'JSON Decoding Error',
    799: 'Unexpected Error'
}

tune_reporting_error_description = {
    -1: 'Unassiged exit condition',
    0: 'Successfully completed',
    700: 'Reporting error occurred',
    707: 'Unexpected request failure',
    708: 'Unexpected software error was detected',
    710: 'Unexpected value returned',
    721: 'Job Stopped',
    722: 'Retry Exhausted',
    760: 'Auth Error',
    761: 'Auth JSON Error',
    766: 'JSON Decoding Error',
    799: 'Unexpected Error'
}


def error_name(error_code):
    """Provide definition of Error Code

    Args:
        error_code:

    Returns:

    """
    if error_code is None or not isinstance(error_code, int):
        return "Error Code: Invalid Type: {}".format(error_code)

    error_code_name_ = tune_requests_error_name(error_code, return_bool=True)
    if error_code_name_ and error_code_name_ is not None:
        return error_code_name_

    error_code_name_ = tune_reporting_error_name.get(error_code, None)
    if error_code_name_ is not None:
        return error_code_name_

    return "Error Code: Undefined: {}".format(error_code)


def error_desc(error_code):
    """Provide definition of Error Code

    Args:
        error_code:

    Returns:

    """
    if error_code is None or not isinstance(error_code, int):
        return "Error Code: Invalid Type: {}".format(error_code)

    error_code_description_ = tune_requests_error_desc(error_code, return_bool=True)
    if error_code_description_ and error_code_description_ is not None:
        return error_code_description_

    error_code_description_ = tune_reporting_error_description.get(error_code, None)
    if error_code_description_ is not None:
        return error_code_description_

    return "Error Code: Undefined: {}".format(error_code)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""
TUNE Multiverse Reporting Base
================================
"""

import logging
from pprintpp import pprint

from requests_mv_integrations.exceptions import (TuneRequestBaseError)

from logging_mv_integrations import (LoggingFormat, LoggingOutput)

from tune_reporting.errors import (get_exception_message, print_traceback, TuneReportingErrorCodes)
from tune_reporting.exceptions import (TuneReportingError, TuneReportingAuthError)
from tune_reporting.support import (base_class_name)
from tune_reporting.tmc.v2.management.tmc_v2_advertisers import (TuneV2Advertisers)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (TuneV2AuthenticationTypes)

log = logging.getLogger(__name__)


def tmc_auth_v2_advertiser(
    tmc_api_key,
    logger_level=logging.NOTSET,
    logger_format=LoggingFormat.JSON,
    logger_output=LoggingOutput.STDOUT_COLOR
):
    """TMC Authentication

    :return:
    """
    log.info("TMC v2 Advertiser: Authentication: Start")

    response_auth = None

    tune_v2_advertisers = \
        TuneV2Advertisers(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

    try:
        try:
            if tune_v2_advertisers.get_advertiser_id(
                auth_type=TuneV2AuthenticationTypes.API_KEY,
                auth_value=tmc_api_key,
                request_retry=None
            ):
                advertiser_id = tune_v2_advertisers.advertiser_id

                log.debug("TMC v2 Advertiser: {}".format(advertiser_id))

        except TuneRequestBaseError as tmc_req_ex:
            pprint(tmc_req_ex.to_dict())
            print(str(tmc_req_ex))

        except TuneReportingError as tmc_rep_ex:
            pprint(tmc_rep_ex.to_dict())
            print(str(tmc_rep_ex))

        except Exception as ex:
            print_traceback(ex)
            print(get_exception_message(ex))

    except Exception as ex:
        print_traceback(ex)
        error_code = TuneReportingErrorCodes.REP_ERR_SOFTWARE

        log.error(
            'TMC v2 Advertiser: Authentication: Failed: Unexpected',
            extra={
                'exit_code': error_code,
                'error_exception': base_class_name(ex),
                'error_details': get_exception_message(ex)
            }
        )

        raise TuneReportingAuthError(
            error_message="TMC v2 Advertiser: Authentication: Failed: Unexpected", errors=ex, error_code=error_code
        )

    if response_auth:
        log.debug("TMC v2 Advertiser: Authentication: Details", extra=response_auth.to_dict())

    return response_auth



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertiser Sites.
"""

import sys
import logging

from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (
    print_traceback,
    get_exception_message,
    TuneReportingErrorCodes
)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)
from tune_reporting import (__python_required_version__)
from tune_reporting.tmc.tune_mobileapptracking_api import (TuneMobileAppTrackingApi)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (TuneV2AuthenticationTypes)
from tune_reporting.tmc.tmc_auth_v2_session_token import (tmc_auth_v2_session_token)
from logging_mv_integrations import (
    LoggingFormat,
    LoggingOutput
)

python_check_version(__python_required_version__)


# @brief TUNE Advertiser Sites.
#
# @namespace tune_reporting.TuneV2AdvertiserSiteStatus
class TuneV2AdvertiserSiteStatus(object):
    """Status states for Advertiser Sites.
    """
    ACTIVE = 1
    NOT_ACTIVE = 2
    ALL = 3


#   TMC v2 Advertiser Sites
#
class TuneV2AdvertiserSites(TuneMobileAppTrackingApi):
    """TMC v2 Advertiser Sites
    """

    # Initialize Job
    #
    def __init__(
        self,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR
    ):
        super(TuneV2AdvertiserSites, self).__init__(
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

    # Collect TMC v2 Advertiser Sites
    #
    def collect(
        self,
        auth_value,
        auth_type,
        auth_type_use,
        site_ids=None,
        request_params=None,
        site_status=TuneV2AdvertiserSiteStatus.ACTIVE
    ):
        """Collect TMC v2 Advertiser Sites

        Args:
            tmc_api_key:
            site_ids:
            request_params:
            request_retry:
            site_status:

        Returns:

        """
        if not auth_value:
            raise ValueError("TuneV2AdvertiserSites: Collect: Value 'auth_value' not provided.")
        if not auth_type:
            raise ValueError("TuneV2AdvertiserSites: Collect: Value 'auth_type' not valid.")
        if not auth_type_use or \
                not TuneV2AuthenticationTypes.validate(auth_type_use):
            raise ValueError("TMC v3 Logs Advertisers Base: Collect: Value 'auth_type_use' not valid.")

        _request_params = {"source": "multiverse"}

        if auth_type == TuneV2AuthenticationTypes.API_KEY:
            self.api_key = auth_value

            if auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
                self.session_token = tmc_auth_v2_session_token(
                    tmc_api_key=self.api_key, logger_level=self.logger_level, logger_format=self.logger_format
                )

                _request_params.update({"session_token": self.session_token,})
            else:
                _request_params.update({"api_key": self.api_key,})

        elif auth_type == TuneV2AuthenticationTypes.SESSION_TOKEN:
            self.session_token = auth_value

            _request_params.update({"session_token": self.session_token,})
        else:
            raise ValueError("Invalid 'auth_type': '{}'".format(auth_type))

        request_url = \
            self.tune_mat_request_path(
                mat_api_version="v2",
                controller="advertiser/sites",
                action="find"
            )

        self.logger.info("Start Advertiser Sites find")

        filter_status = None
        if site_status == TuneV2AdvertiserSiteStatus.ACTIVE:
            filter_status = "(status = \"active\")"
        elif site_status == TuneV2AdvertiserSiteStatus.NOT_ACTIVE:
            filter_status = "(status != \"active\")"

        filter_sites = None
        if site_ids and len(site_ids) > 0:
            filter_sites = "(id IN (" + ",".join(site_ids) + "))"

        if filter_status and filter_sites:
            _request_params["filter"] = "{} AND {}".format(filter_status, filter_sites)
        elif filter_status:
            _request_params["filter"] = filter_status
        elif filter_sites:
            _request_params["filter"] = filter_sites

        if request_params:
            if "filter" in request_params and request_params["filter"]:
                if "filter" in _request_params and _request_params["filter"]:
                    _request_params["filter"] = "({} AND {})".format(
                        _request_params["filter"], request_params["filter"]
                    )
                else:
                    _request_params["filter"] = "({})".format(request_params["filter"])
            _request_params.update({'limit': request_params.get('limit', 0)})
        else:
            _request_params.update({'limit': 0})

        response = None
        try:
            response = self.mv_request.request(
                request_method="GET",
                request_url=request_url,
                request_params=_request_params,
                request_retry=None,
                request_retry_http_status_codes=None,
                request_retry_func=self.tune_v2_request_retry_func,
                request_retry_excps_func=None,
                request_label="{}.{}".format(self.__class__.__name__, sys._getframe().f_code.co_name)
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TuneV2AdvertiserSites: Collect: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            yield (None, tmc_req_ex)

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TuneV2AdvertiserSites: Collect: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            yield (None, tmc_rep_ex)

        except Exception as ex:
            print_traceback(ex)

            self.logger.error(get_exception_message(ex))
            yield (None, ex)

        if response:
            json_response = response.json()
            if not json_response or \
                    json_response['status_code'] != 200 or \
                    'errors' in json_response:
                raise TuneReportingError(
                    error_message="Failed to get advertiser sites: {}, {}".format(
                        json_response['status_code'],
                        json_response['errors'].get('message', None),
                    ),
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
                )

            if ('data' not in json_response or not json_response['data']):
                raise TuneReportingError(
                    error_message="Missing 'data': {}".format(str(json_response)),
                    error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
                )

            data = json_response['data']
        else:
            data = None

        if not data or len(data) == 0:
            yield ([], None)
        else:
            for (i, item) in enumerate(data):
                yield (item, None)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from tune_reporting.tmc.v2.management.tmc_v2_advertiser_sites import (
    TuneV2AdvertiserSites,
    TuneV2AdvertiserSiteStatus
)
from tune_reporting.tmc.v2.management.tmc_v2_advertisers import (TuneV2Advertisers)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (
    TuneV2SessionAuthenticate,
    TuneV2AuthenticationTypes
)
from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_actuals import (
    TuneV2AdvertiserStatsActuals
)
from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_base import (
    TuneV2AdvertiserStatsActions,
    TuneV2AdvertiserStatsFormats
)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_base import (
    TuneV3LogsAdvertisersActions
)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_clicks import (
    TuneV3LogsAdvertisersClicks
)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_impressions import (
    TuneV3LogsAdvertisersImpressions
)
from .tune_mobileapptracking_api_base import (
    TuneMobileAppTrackingApiBase
)
from .tmc_auth_v2_advertiser import (tmc_auth_v2_advertiser)
from .tmc_auth_v2_session_token import (tmc_auth_v2_session_token)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

__title__ = 'tune-reporting'
__version__ = '2.5.4'
__version_info__ = tuple(__version__.split('.'))

__author__ = 'jefft@tune.com'
__license__ = 'The MIT License (MIT)'
__copyright__ = 'Copyright 2017 TUNE, Inc.'

__python_required_version__ = (3, 0)

from tune_reporting.tmc.v2.management.tmc_v2_advertiser_sites import (TuneV2AdvertiserSites, TuneV2AdvertiserSiteStatus)
from tune_reporting.tmc.v2.management.tmc_v2_advertisers import (TuneV2Advertisers)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (
    TuneV2SessionAuthenticate, TuneV2AuthenticationTypes
)
from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_actuals import (TuneV2AdvertiserStatsActuals)
from tune_reporting.tmc.v2.reporting.tmc_v2_advertiser_stats_base import (
    TuneV2AdvertiserStatsActions, TuneV2AdvertiserStatsFormats
)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_base import (TuneV3LogsAdvertisersActions)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_clicks import (TuneV3LogsAdvertisersClicks)
from tune_reporting.tmc.v3.reporting.tmc_v3_logs_advertisers_impressions import (TuneV3LogsAdvertisersImpressions)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""Report Reader CSV
"""

from logging import getLogger
import sys
import csv
import requests

# from pprintpp import pprint

from .report_reader_base import (ReportReaderBase)

log = getLogger(__name__)


# @brief TUNE Multiverse Report CSV reader.
#
# @namespace tune_reporting.ReportReaderCSV
class ReportReaderCSV(ReportReaderBase):
    """Helper class for reading reading remote CSV file
    """

    #  The constructor
    #  @param str report_url Download report URL
    #                         of requested report to be exported.
    def __init__(self, report_url):
        """The constructor.

            :param str report_url: Report URL to be downloaded.
        """
        if not report_url:
            raise ValueError(error_message="Undefined 'report_url'")
        self.reader = None
        super(ReportReaderCSV, self).__init__(report_url)

    #  Using provided report download URL, extract CSV contents.
    #
    def read(self, csv_delimiter=','):
        """Read CSV data provided remote path report_url.

        Args:
            csv_delimiter:

        Returns:

        """
        self.data = None

        response = requests.get(url=self.report_url, stream=True)

        log.info("ReportReaderCSV: Response", extra={'http_status_code': response.status_code})

        data = response.text

        reader = csv.DictReader(data.splitlines(), delimiter=csv_delimiter)

        self.data = list(reader)
        self.count = len(self.data)

    def next(self):
        """Retrieve the next item from the iterator."""
        try:
            row = self.reader.__next__()
            return [str(value, "utf-8") for value in row]

        except StopIteration:
            pass

        return None

    def __iter__(self):
        """This method is called when an iterator is required for a container.
        """
        # pylint: disable=not-an-iterable
        for row in self.reader:
            yield row
        # pylint: enable=not-an-iterable

    def pretty_print(self, limit=0):
        """Pretty print exported data.

            :param int limit: Number of rows to print.
        """
        print("Report REPORT_URL: {}".format(self.report_url))
        print("------------------")

        if sys.version_info >= (3, 0, 0):
            # pylint: disable=not-an-iterable
            for row in self.reader:
                print(', '.join(row))
            # pylint: enable=not-an-iterable
        else:
            i = 0
            while True:
                row = self.next()
                if row is None:
                    break
                i += 1
                print("{}. {}".format(i, str(row)))
                if (limit > 0) and (i >= limit):
                    break

        print("------------------")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertiser Stats Clicks.
"""

import logging
# from pprintpp import pprint

from pytz_convert import (validate_tz_name)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from tune_reporting.errors import (print_traceback, get_exception_message)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)

from .tmc_v3_logs_advertisers_base import (
    TuneV3LogsAdvertisersBase,
    TuneV3LogsAdvertisersActions
)
from tune_reporting import (__python_required_version__)
from tune_reporting.tmc.v2.management.tmc_v2_session_authenticate import (
    TuneV2AuthenticationTypes
)
from logging_mv_integrations import (
    LoggingFormat,
    LoggingOutput
)

python_check_version(__python_required_version__)


# @brief TUNE Advertiser Stats Clicks.
#
# @namespace tune_reporting.TuneV3LogsAdvertisersImpressions
class TuneV3LogsAdvertisersImpressions(TuneV3LogsAdvertisersBase):
    """TUNE V3 Logs Advertiser Impressions.
    """

    _LOGS_ADVERTISERS_TYPE = "impressions"

    # Initialize Job
    #
    def __init__(
        self,
        timezone=None,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR
    ):
        """Initialize TUNE V3 Logs Advertiser Impressions class.
        """
        super(TuneV3LogsAdvertisersImpressions, self).__init__(
            logs_advertisers_type=self._LOGS_ADVERTISERS_TYPE,
            timezone=timezone,
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output
        )

    # Collect data: TUNE V3 Logs Advertiser Impressions.
    #
    def _collect(
        self,
        auth_type_use,
        start_date,
        end_date,
        request_params=None,
        request_retry=None,
        request_action=TuneV3LogsAdvertisersActions.FIND,
        auth_type=TuneV2AuthenticationTypes.API_KEY
    ):
        """Collect data: TUNE V3 Logs Advertiser Impressions.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:
            request_action:

        Returns:

        """
        auth_value = None
        if auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            auth_value = self.api_key
        elif auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            auth_value = self.session_token

        dict_request_params = {
            auth_type_use: auth_value,
            "timezone": self.timezone,
            "fields": (
                "created,"
                "ad_network_id,"
                "campaign.id,"
                "campaign.name,"
                "publisher.id,"
                "publisher.name,"
                "publisher_ref_id,"
                "publisher_sub_site.id,"
                "publisher_sub_site.ref,"
                "publisher_sub_site.name,"
                "publisher_sub_campaign.id,"
                "publisher_sub_campaign.ref,"
                "publisher_sub_campaign.name,"
                "request_url,"
                "site.id,"
                "site.mobile_app_type,"
                "site.name,"
                "site.package_name,"
                "site.store_app_id"
            ),
            "filter": "({})".format(self._FILTER_NOT_DEBUG_NOR_TEST_DATA),
            "start_date": start_date,
            "end_date": end_date,
            "debug": 0
        }

        if request_params:
            if "fields" in request_params and request_params["fields"]:
                dict_request_params["fields"] = \
                    request_params["fields"]

            if "group" in request_params and request_params["group"]:
                dict_request_params["group"] = \
                    request_params["group"]

            if "timestamp" in request_params and request_params["timestamp"]:
                dict_request_params["timestamp"] = \
                    request_params["timestamp"]

            if "filter" in request_params and request_params["filter"]:
                dict_request_params["filter"] = "({} AND {})".format(
                    request_params["filter"], self._FILTER_NOT_DEBUG_NOR_TEST_DATA
                )

            if "limit" in request_params:
                dict_request_params["limit"] = \
                    int(request_params["limit"])

            if "debug" in request_params:
                dict_request_params["debug"] = \
                    int(request_params["debug"])

            timezone = None
            if "timezone" in request_params:
                timezone = request_params["timezone"]

            if timezone:
                if not validate_tz_name(timezone):
                    return TuneReportingError(error_message="Invalid Timezone: {}".format(timezone))
                self.timezone = timezone
                dict_request_params["timezone"] = \
                    self.timezone

        self.logger.debug(("TMC v3 Logs Advertisers Impressions: "
                           "Action '{}', Params: {}").format(request_action, str(dict_request_params)))

        self.logger.debug(("TMC v3 Logs Advertisers Impressions: " "Timezone: {}").format(self.timezone))

        try:
            if request_action == TuneV3LogsAdvertisersActions.FIND:

                request_params["sorts"] = "created desc"

                self._find_v3(request_params=dict_request_params, request_retry=request_retry)
            elif request_action == TuneV3LogsAdvertisersActions.EXPORT:
                self._export_v3_download_csv(request_params=dict_request_params, request_retry=request_retry)

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Impressions: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TMC v3 Logs Advertisers Impressions: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TMC v3 Logs Advertisers Impressions: Failed: {}".format(get_exception_message(ex)))

            raise TuneReportingError(
                error_message=("TMC v3 Logs Advertisers Impressions: Failed: {}").format(get_exception_message(ex)),
                errors=ex
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""TUNE Advertiser Stats Actuals.
"""

import logging
#from pprintpp import pprint
from pytz_convert import (validate_tz_name)
from requests_mv_integrations.exceptions import (TuneRequestBaseError)
from safe_cast import safe_dict
from tune_reporting.errors import (
    print_traceback,
    get_exception_message,
    TuneReportingErrorCodes
)
from tune_reporting.exceptions import (TuneReportingError)
from tune_reporting.support import (python_check_version)
from .tmc_v2_advertiser_stats_base import (
    TuneV2AdvertiserStatsBase,
    TuneV2AdvertiserStatsActions,
    TuneV2AuthenticationTypes,
)
from tune_reporting import (__python_required_version__)
from logging_mv_integrations import (LoggingFormat, LoggingOutput)
log = logging.getLogger(__name__)

python_check_version(__python_required_version__)


# @brief TUNE Advertiser Stats Actuals.
#
# @namespace tune_reporting.TuneV2AdvertiserStatsActuals
class TuneV2AdvertiserStatsActuals(TuneV2AdvertiserStatsBase):
    """TMC v2 Advertiser Stats.
    """

    _CONTROLLER = "advertiser/stats"

    # Initialize Job
    #
    def __init__(
        self,
        logger_level=logging.NOTSET,
        logger_format=LoggingFormat.JSON,
        logger_output=LoggingOutput.STDOUT_COLOR,
        timezone=None
    ):
        """Initialize TMC v2 Advertiser Stats Class.
        """
        super(TuneV2AdvertiserStatsActuals, self).__init__(
            controller=self._CONTROLLER,
            logger_level=logger_level,
            logger_format=logger_format,
            logger_output=logger_output,
            timezone=timezone
        )

    # Collect data: TMC v2 Advertiser Stats Actuals.
    #
    def _collect(
        self,
        auth_type_use,
        start_date,
        end_date,
        tmp_directory=None,
        request_params=None,
        request_retry=None,
        request_action=TuneV2AdvertiserStatsActions.FIND
    ):
        """Collect data: TUNE Advertiser Stats Actuals.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:
            request_action:

        Returns:

        """
        self.logger.debug(("TuneV2AdvertiserStatsActuals: Collect: " "Action: '{}'").format(request_action))
        dict_request_params = self._map_request_params(auth_type_use, start_date, end_date, request_params)

        self.logger.debug("TuneV2AdvertiserStatsActuals: Collect", extra={'build_params': dict_request_params})

        try:
            if request_action == TuneV2AdvertiserStatsActions.FIND:
                self._find_v2(request_params=dict_request_params, request_retry=request_retry)
            elif request_action == TuneV2AdvertiserStatsActions.EXPORT:
                assert tmp_directory
                self._export_v2_download_csv(
                    tmp_directory=tmp_directory,
                    auth_type_use=auth_type_use,
                    export_controller=self._CONTROLLER,
                    export_action='export',
                    export_status_controller=self._CONTROLLER,
                    export_status_action='status',
                    request_params=dict_request_params,
                    request_retry=request_retry
                )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TuneV2AdvertiserStatsActuals: Collect: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TuneV2AdvertiserStatsActuals: Collect: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TuneV2AdvertiserStatsActuals: Collect: {}".format(get_exception_message(ex)))

            raise TuneReportingError(
                error_message=("TuneV2AdvertiserStatsActuals: Collect: Failed: {}").format(get_exception_message(ex)),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

    # Collect data: TUNE Advertiser Stats Actuals.
    #
    def _stream_v2(
        self,
        auth_type_use,
        start_date,
        end_date,
        request_params,
        request_retry=None,
    ):
        """Stream data: TUNE Advertiser Stats Actuals.

        Args:
            start_date:
            end_date:
            request_params:
            request_retry:

        Returns:

        """
        self.logger.debug("TuneV2AdvertiserStatsActuals: Stream: Export", extra={'request_params': request_params})

        dict_request_params = self._map_request_params(auth_type_use, start_date, end_date, request_params)

        self.logger.debug("TuneV2AdvertiserStatsActuals: Stream: Export", extra={'build_params': dict_request_params})

        try:
            response = self._export_stream_v2(
                auth_type_use,
                export_controller=self._CONTROLLER,
                export_action='export',
                export_status_controller=self._CONTROLLER,
                export_status_action='status',
                request_params=dict_request_params,
                request_retry=request_retry
            )

        except TuneRequestBaseError as tmc_req_ex:
            self.logger.error(
                "TuneV2AdvertiserStatsActuals: Stream: Failed",
                extra=tmc_req_ex.to_dict(),
            )
            raise

        except TuneReportingError as tmc_rep_ex:
            self.logger.error(
                "TuneV2AdvertiserStatsActuals: Stream: Failed",
                extra=tmc_rep_ex.to_dict(),
            )
            raise

        except Exception as ex:
            print_traceback(ex)

            self.logger.error("TuneV2AdvertiserStatsActuals: {}".format(get_exception_message(ex)))

            raise TuneReportingError(
                error_message=("TuneV2AdvertiserStatsActuals: Failed: {}").format(get_exception_message(ex)),
                errors=ex,
                error_code=TuneReportingErrorCodes.REP_ERR_SOFTWARE
            )

        return response

    def _map_request_params(self, auth_type_use, start_date, end_date, request_params=None):
        """Build Request Paramaters

        Args:
            start_date:
            end_date:
            request_params:

        Returns:

        """
        auth_value = None
        if auth_type_use == TuneV2AuthenticationTypes.API_KEY:
            auth_value = self.api_key
        elif auth_type_use == TuneV2AuthenticationTypes.SESSION_TOKEN:
            auth_value = self.session_token

        dict_request_params = {
            auth_type_use: auth_value,
            "source": "multiverse",
            "response_timezone": self.timezone,
            "timestamp": "datehour",
            "group": (
                "advertiser_id,"
                "country_id,"
                "currency_code,"
                "is_reengagement,"
                "platform,"
                "publisher_id,"
                "publisher_sub_ad_id,"
                "publisher_sub_adgroup_id,"
                "publisher_sub_campaign_id,"
                "publisher_sub_publisher_id,"
                "publisher_sub_site_id,"
                "purchase_validation_status,"
                "site_id"
            ),
            "fields": (
                "ad_clicks,"
                "ad_clicks_unique,"
                "ad_impressions,"
                "ad_impressions_unique,"
                "ad_network_id,"
                "advertiser_id,"
                "conversions,"
                "country.code,"
                "country.name,"
                "currency_code,"
                "date_hour,"
                "events,"
                "installs,"
                "is_reengagement,"
                "payouts,"
                "publisher.name,"
                "publisher_id,"
                "publisher_sub_ad.ref,"
                "publisher_sub_adgroup.ref,"
                "publisher_sub_campaign.ref,"
                "publisher_sub_publisher.ref,"
                "publisher_sub_site.ref,"
                "site.mobile_app_type,"
                "site.package_name,"
                "site.store_app_id,"
                "site_id"
            ),
            "filter": "({})".format(self._FILTER_NOT_DEBUG_NOR_TEST_DATA),
            "start_date": start_date,
            "end_date": end_date,
            "debug": 0
        }

        if request_params:
            self.logger.debug(
                "TuneV2AdvertiserStatsActuals: Request", extra={'request_params': safe_dict(request_params)}
            )

            if "fields" in request_params and \
                    request_params["fields"]:
                dict_request_params["fields"] = \
                    request_params["fields"]

            if "group" in request_params and \
                    request_params["group"]:
                dict_request_params["group"] = \
                    request_params["group"]

            if "timestamp" in request_params and \
                    request_params["timestamp"]:
                dict_request_params["timestamp"] = \
                    request_params["timestamp"]

            if "filter" in request_params and \
                    request_params["filter"]:
                dict_request_params["filter"] = "({} AND {})".format(
                    request_params["filter"], self._FILTER_NOT_DEBUG_NOR_TEST_DATA
                )

            if "format" in request_params:
                dict_request_params["format"] = \
                    request_params["format"]

            if "offset" in request_params:
                dict_request_params["offset"] = \
                    int(request_params["offset"])

            if "page" in request_params:
                dict_request_params["page"] = \
                    int(request_params["page"])

            if "limit" in request_params:
                dict_request_params["limit"] = \
                    int(request_params["limit"])

            if "debug" in request_params:
                dict_request_params["debug"] = \
                    int(request_params["debug"])

            response_timezone = None
            if "timezone" in request_params:
                response_timezone = request_params["timezone"]
            if "response_timezone" in request_params:
                response_timezone = request_params["response_timezone"]

            if response_timezone:
                if not validate_tz_name(response_timezone):
                    return TuneReportingError(error_message="Invalid Timezone: {}".format(response_timezone))
                self.timezone = response_timezone
                dict_request_params["response_timezone"] = \
                    self.timezone

        self.logger.debug(("TuneV2AdvertiserStatsActuals: " "Timezone: {}").format(self.timezone))

        return dict_request_params



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting

from .reporting.tmc_v3_logs_advertisers_base import (TuneV3LogsAdvertisersActions)
from .reporting.tmc_v3_logs_advertisers_clicks import (TuneV3LogsAdvertisersClicks)
from .reporting.tmc_v3_logs_advertisers_impressions import (TuneV3LogsAdvertisersImpressions)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace requests_mv_integrations
"""Helpers: Functions for commonly used utilities.
"""

import sys


#  Check Python Version
#
def python_check_version(required_version):
    """Check Python Version
    :param: required_version
    """
    current_version = sys.version_info
    if current_version[0] == required_version[0] and \
       current_version[1] >= required_version[1]:
        pass
    elif current_version[0] > required_version[0]:
        pass
    else:
        sys.stderr.write(
            "[%s] - Error: Python interpreter must be %d.%d or greater"
            " to use this library, current version is %d.%d.\n" %
            (sys.argv[0], required_version[0], required_version[1], current_version[0], current_version[1])
        )
        sys.exit(-1)
    return 0


def convert_size(size, precision=2):
    """Convert Size

    Args:
        size:
        precision:

    Returns:

    """
    suffixes = ['B', 'KB', 'MB', 'GB', 'TB']
    suffixIndex = 0

    while size > 1024 and suffixIndex < 4:
        suffixIndex += 1  # increment the index of the suffix
        size = size / 1024.0  # apply the division

    suffix = suffixes[suffixIndex]
    if suffix == 'B':
        precision = 0

    return "%.*f %s" % (precision, size, suffix)


def base_class_name(obj):
    return obj.__class__.__name__


def full_class_name(obj):
    try:
        return obj.__module__ + "." + obj.__class__.__name__
    except Exception as ex:
        return obj.__class__.__name__



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace tune_reporting
"""
Tune Reporting Error
"""

from tune_reporting.errors import (TuneReportingErrorCodes)
from .base import (TuneReportingBaseError)


class TuneReportingError(TuneReportingBaseError):
    pass


class TuneReportingAuthError(TuneReportingBaseError):
    def __init__(self, **kwargs):
        error_code = kwargs.pop('error_code', None) or TuneReportingErrorCodes.REP_ERR_AUTH_ERROR
        super(TuneReportingAuthError, self).__init__(error_code=error_code, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @copyright 2017 TUNE, Inc. (http://www.tune.com)
#  @namespace requests_mv_integrations.errors
"""Error handling tools.
"""

import sys
import traceback


def get_exception_message(ex):
    """Build exception message with details.
    """
    template = "{0}: {1!r}"
    return template.format(type(ex).__name__, ex.args)


def print_traceback(ex):
    exc_type, exc_value, exc_tb = sys.exc_info()
    traceback.print_exception(exc_type, exc_value, exc_tb)


def print_limited_traceback(ex, limit=1):
    exc_type, exc_value, exc_tb = sys.exc_info()
    traceback.print_exc(limit=1, file=sys.stdout)


def print_traceback_stack():
    """Provide traceback of provided exception.
    """
    exception_list = traceback.format_stack()
    exception_list = exception_list[:-2]
    exception_list.extend(traceback.format_tb(sys.exc_info()[2]))
    exception_list.extend(traceback.format_exception_only(sys.exc_info()[0], sys.exc_info()[1]))

    exception_str = "Traceback (most recent call last):\n"
    exception_str += "".join(exception_list)
    # Removing the last \n
    exception_str = exception_str[:-1]

    print(exception_str)
